//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.cannery;

import java.io.*;
import java.util.*;


class CanneryProperties extends Properties {

String propFileName = "cannery.properties";

String propDebug      = "Debug";
String propVerbose    = "Verbose";
String propCompress   = "Compress";
String propExclude    = "Exclude";
String propScanDeps   = "ScanDeps";
String propAddSources = "AddSources";
String propUseSHA     = "UseSHADigest";
String propUseMD5     = "UseMD5Digest";


private Properties excludes = new Properties();

public CanneryProperties() { init(); }

public void init() {
    defaults = new Properties();
    try {
        FileInputStream in = new FileInputStream(propFileName);
        defaults.load(in);
        in.close();
    } catch (FileNotFoundException e) {
        defaults.put(propVerbose,    "true");
        defaults.put(propCompress,   "true");
        defaults.put(propExclude,    "java");
        defaults.put(propScanDeps,   "true");
        defaults.put(propAddSources, "false");
        defaults.put(propUseSHA,     "true");
        defaults.put(propUseMD5,     "false");
        try {
            FileOutputStream out = new FileOutputStream(propFileName);
            defaults.save(out, " Properties for Java Cannery");
            out.close();
        } catch (IOException x) {}
    } catch (IOException e) {System.err.println("Couldn't load properties file "+propFileName); }
    excludes.clear();
    String exclude;
    if ((exclude = getProperty(propExclude)) != null) {
        parseExcludes(exclude);
    }
}
void parseExcludes(String exclude) {
    StringTokenizer tok = new StringTokenizer(exclude, " \t;,", false);
    while (tok.hasMoreTokens()) {
        String s=tok.nextToken();
        excludes.put(s, s);
    }
}
void exclude(String exclude) { parseExcludes(exclude); }

void include(String include) {
  StringTokenizer tok = new StringTokenizer(include, " \t;,", false);
  while (tok.hasMoreTokens()) {
      String s=tok.nextToken();
      excludes.remove(s);
  }
}
boolean isExcluded(String s) { return excludes.containsKey(s); }
public Enumeration excluded() { return excludes.keys(); }

void setDebug(boolean d) { put(propDebug, (d? "true": "false")); }
boolean isDebug() { String s=getProperty(propDebug, "false"); return s.equals("true"); }

void setVerbose(boolean v) { put(propVerbose, (v? "true": "false")); }
boolean isVerbose() { String s=getProperty(propVerbose, "true"); return s.equals("true"); }

void setCompress(boolean c) { put(propCompress, (c? "true": "false")); }
boolean getCompress() { String s=getProperty(propCompress, "false"); return s.equals("true"); }

void setAddSources(boolean c) { put(propAddSources, (c? "true": "false")); }
boolean shouldAddSources() { String s=getProperty(propAddSources, "false"); return s.equals("true"); }

void setUseSHADigest(boolean c) { put(propUseSHA, (c? "true": "false")); }
boolean shouldUseSHADigest() { String s=getProperty(propUseSHA, "true"); return s.equals("true"); }

void setUseMD5Digest(boolean c) { put(propUseMD5, (c? "true": "false")); }
boolean shouldUseMD5Digest() { String s=getProperty(propUseMD5, "true"); return s.equals("true"); }



static public void main(String[] args) {
    CanneryProperties props = new CanneryProperties();
    for (int argn=0; argn<args.length; ++argn) {
        if ( "-loud".startsWith(args[argn]) )
            props.setVerbose(true);
        else if ( "-quiet".startsWith(args[argn]) )
            props.setVerbose(false);
        else if ( "-compress".startsWith(args[argn]) )
            props.setCompress(true);
        else if ( "-nocompress".startsWith(args[argn]) )
            props.setCompress(false);
        else if ( "-exclude".startsWith(args[argn]) )
            props.exclude(args[++argn]);
    }
    System.out.println("Defaults:");
    props.defaults.list(System.out);

    System.out.println("Properties:");
    props.list(System.out);
}
public void list(PrintStream out) {
    super.list(out);
    out.println("--excludes:--");
    excludes.list(out);
}
public void list(PrintWriter out) {
    super.list(out);
    out.println("--excludes:--");
    excludes.list(out);
}
}
