//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.cannery;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.text.*;
import starlight.ui.*;
import starlight.util.*;
import starlight.java.*;


public class JarTreeView extends TreePanel implements Observer
{
private JarArchive jar;

public JarTreeView() {}

public void update(Observable o, Object arg) {
    //Msg.notify("starting update.");
    long start = System.currentTimeMillis();
    if (o instanceof JarArchive) {
        if (o != jar)              // somebody switched jars
            jar = (JarArchive) o;

        //Msg.notify("getting jar contents...");
        Vector vec = new Vector();
        for (Enumeration e=jar.childNames(); e.hasMoreElements(); ) {
            String path = (String) e.nextElement();
            vec.addElement(path);
        }
        String[] files = new String[vec.size()];
        for (int i=0; i<files.length; i++)
            files[i] = (String) vec.elementAt(i);

        clear();

        //Msg.notify("adding elements...");
        setRoot(jar.getName(), Resources.RootIcon);

        for (int i=0; i<files.length; i++)
            addItem(files[i]);
    }
    //Msg.notify("validating...");
    invalidate(); validate(); repaint();
    long end = System.currentTimeMillis();
    //Msg.notify("JarTreeView updated.  ("+(end-start)+" ms)");
}

    public Branch addBranch(Branch parent, String label)
        {   return super.addBranch(parent, label, Resources.OpenIcon, Resources.ClosedIcon);  }

    public Leaf addLeaf(Branch parent, String label)
        {   return super.addLeaf(parent, label, null);  }

    void addItem(String qualname) {
        int ix = qualname.lastIndexOf(jar.separatorChar);
        String path = (ix >= 0) ? qualname.substring(0, ix+1): "";
        String name = (ix >= 0) ? qualname.substring(ix+1): qualname;
        Branch parent = root;
        if (path.length() > 0) parent = force(path);
        if (name.length() > 0)
            if (jar.isDirectory(qualname))
                addBranch(parent, name);
            else  addLeaf(parent, name);
    }
    void removeItem(String qualname) {
        int ix = qualname.lastIndexOf(jar.separatorChar, qualname.length()-2);
        String path = (ix >= 0) ? qualname.substring(0, ix+1): "";
        String name = (ix >= 0) ? qualname.substring(ix+1): qualname;
        Branch parent = root;
        if (path.length() > 0) parent = getBranch(path);
        if (name.length() > 0) parent.removeChild(name);
    }
    Branch force(String path) {
        Branch node = root;
        while (path != null && path.length() > 0) {
            String dir = path;
            int ix = path.indexOf(jar.separatorChar);
            if (ix > 0) {
                dir  = path.substring(0, ix+1);
                path = path.substring(ix+1);
            } else path = null;
            Branch ch = (Branch) node.getChild(dir);
            if (ch == null)
                ch = addBranch(node, dir);
            node = ch;
        }
        return node;
    }
    Branch getBranch(String path) {
        Branch node = root;
        while (node != null && path != null && path.length() > 0) {
            String dir = path;
            int ix = path.indexOf(jar.separatorChar);
            if (ix > 0) {
                dir  = path.substring(0, ix+1);
                path = path.substring(ix+1);
            } else path = null;
            node = (Branch) node.getChild(dir);
        }
        return node;
    }

    // leaf may be:
    // o  label of a Branch (it's getLabel() will == the Branch's getLabel(),
    //      since the leaf-label is the branch's first child)
    // o  a resource entry name, with a Branch parent.

    String getQualifiedName(Leaf item) {
        String qualname = "";
        Branch parent = (Branch)item.getParent();//all Leaf have Branch parents

        if (item.getLabel()!=parent.getLabel()) {

            // it's not a branch-label, so we need to append its name
            // (if it were a branch-label, then getParent().getLabel()
            // would return its own label, causing it to be appended
            // twice, which would be (and was) a bug).
            qualname = item.getLabel();
        }

        // work our way up to the root, stopping before we get there.
        // (don't want the root's name(ex. c:\java\some.zip) included)
        while (parent != root) {

            // prepend the parent's name to our string
            qualname = parent.getLabel() + qualname;

            // root's parent will be the Container, but we're not
            // going that far.  Within the tree, all parents are Branches.
            parent = (Branch) parent.getParent();
        }
        return qualname;
    }
    /** translate selection into qualified path names (relative to root) */
    public Object[] getSelectedObjects() {
        Object[] selected = super.getSelectedObjects();
        if (selected == null)  return null;
        String[] names = new String[selected.length];
        for (int i=0; i<names.length; i++)
            names[i] = getQualifiedName((Leaf)selected[i]);
        return names;
    }

//
// scheme originally was to sort entire jar list first, then create
// the tree, and have all the subtrees therefore be sorted.  On large
// datasets, though, this is slow.  The sort is fine; it's Quicksort.
// The problem is in the collation stuff.  Normal string-compare for
// less() would be fairly satisfactory, but I don't like the fact that
// it gives a non-lexicographical sorting (all the capitalized entries
// come first, followed by all the lower-case ones, instead of having
// them properly mingled (Aardvark, abacus, Belgian...)).  The collation
// classes fix that, however the getCollationKey(String) method takes
// too much time (several seconds to do a few thousand entries).
//  So, it's yanked.  Sorting now takes place in the individual branches,
// on demand (when a non-sorted branch is laid out).  As a benefit of
// that method, I was able to use the Collation class, and therefore
// sorting should work properly across all locallizations.
//  This is left in for informational purposes only.
//
private void sort(String[] list) {
    Msg.notify("getting collation keys...");
    Collator collator = Collator.getInstance();
    CollationKey[] keys = new CollationKey[list.length];
    for (int i=0; i<list.length; i++)
        keys[i] = collator.getCollationKey(list[i]);  //slow!!

    Msg.notify("ArraySorter is sorting...");
    ArraySorter.sort(keys, new Comparator() {
        public boolean less(Object lhs, Object rhs)
        { return (((CollationKey)lhs).compareTo((CollationKey)rhs) < 0); }
    });
    Msg.notify("extracting dirs...");
    int marker = 0;
    for (int i=0; i<keys.length; i++) {         // pull out the dirs first...
        String filename = keys[i].getSourceString();
        if (jar.isDirectory(filename)) {
            list[marker++] = filename;
            keys[i] = null;
    }   }
    for (int i=0; i<keys.length; i++) {        // what's left are the non-dir files.
        if (keys[i] != null)
            list[marker++] = keys[i].getSourceString();
    }
}
}
