//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.cannery;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import starlight.java.*;
import starlight.util.*;
import starlight.ui.*;
import starlight.ui.cmd.*;
import starlight.dialog.MessageBox;
import starlight.dialog.*;

public class MainFrame extends Frame implements Observer, StatusMessageTarget
{
private static String title = "Java Cannery";
private static String version = "0.5";

private String destFileName;
private CanneryProperties props = new CanneryProperties();

private MainMenu       menu = new MainMenu();
private MainToolBar toolbar = new MainToolBar(FlowLayout.LEFT, 6, 6);/*hgap, vgap*/
private Label     statusbar = new Label();
private Panel          view = new JarTreeView();

private JarArchive jar;             // model in MVC
private Object[] selection;         // currently selected objects in view


MainFrame(String title) {
    super(title);

    setLayout(new BorderLayout());
    setFont(Resources.appFont);
    setIconImage(Resources.appIcon);
    setMenuBar(menu);
    add("North", toolbar);
    add("Center", view);
    add("South", statusbar);

    initCommands();

    pack();
    centerWindow(this);
    show();

    // capture status/notify messages to statusbar
    Msg.setTarget(this);

    addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) { System.exit(0); }
    });

    ((ItemSelectable)view).addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent e) {
            selection = e.getItemSelectable().getSelectedObjects();
            updateSelectionWatchers();
        }
    });
}
public void addNotify() {
    super.addNotify();
    Color bg = starlight.Colors.atIntensity(getBackground(), 0.94);
    setBackground(bg);
    toolbar  .setBackground(bg.darker());
    statusbar.setBackground(bg.darker());
}

/**
  *     MAIN MENU:
  *
  *   Jar         Item        Help
  *     New         Add         About
  *     Open        Delete      Contents
  *     Save        View
  *     Save As     Launch
  *     Exit
  *
  */
void initCommands() {
    Command cmd;

    menu.createMenuItem(cmd=new Command(Resources.JarNew));
    toolbar.createButton(cmd);
    jarNew = new CommandHandler(cmd)
        { public Memento execute() {newJar(); return null;} };

    menu.createMenuItem(cmd=new Command(Resources.JarOpen));
    toolbar.createButton(cmd);
    jarOpen = new CommandHandler(cmd)
        { public Memento execute() {openJar(); return null;} };

    menu.createMenuItem(cmd=new Command(Resources.JarSave));
    toolbar.createButton(cmd);
    jarSave = new CommandHandler(cmd) {
        public Memento execute() {saveJar(); return null;}
        public boolean isInvokable() { return ((jar!=null) && jar.isModified()); }
    };

    menu.createMenuItem(cmd=new Command(Resources.JarSaveAs));
    //toolbar.createButton(cmd);
    jarSaveAs = new CommandHandler(cmd) {
        public Memento execute() {saveJarAs(); return null;}
        public boolean isInvokable() { return jar!=null; }
    };

    menu.createMenuItem(cmd=new Command(Resources.ItmAdd));
    toolbar.createButton(cmd);
    itemAdd = new CommandHandler(cmd) {
        public Memento execute() {addItem(); return null;}
        public boolean isInvokable() { return jar!=null; }
    };

    menu.createMenuItem(cmd=new Command(Resources.ItmDelete));
    //toolbar.createButton(cmd);
    itemDelete = new CommandHandler(cmd) {
        public Memento execute() {deleteItem(); return null;}
        public boolean isInvokable() { return isSelection(); }
    };

    menu.createMenuItem(cmd=new Command(Resources.ItmExtract));
    toolbar.createButton(cmd);
    itemExtract = new CommandHandler(cmd) {
        public Memento execute() {extractItem(); return null;}
        public boolean isInvokable() { return isSelection(); }
    };

    cmd=new Command(Resources.ItmView);
    //menu.createMenuItem(cmd);
    //toolbar.createButton(cmd);
    itemView = new CommandHandler(cmd) {
        public Memento execute() {viewItem(); return null;}
        public boolean isInvokable() { return isSelection(); }
    };

    cmd=new Command(Resources.ItmLaunch);
    //menu.createMenuItem(cmd);
    //toolbar.createButton(cmd);
    itemLaunch = new CommandHandler(cmd) {
        public Memento execute() {launchItem(); return null;}
        public boolean isInvokable() { return isSelection(); }
    };

    menu.createMenuItem(cmd=new Command(Resources.JarExit));
    toolbar.createButton(cmd);
    jarExit = new CommandHandler(cmd) {
        public Memento execute() {
            dispatchEvent(new WindowEvent((Window)MainFrame.this, WindowEvent.WINDOW_CLOSING));
            return null;
        }
    };

    menu.createMenuItem(cmd=new Command(Resources.HlpAbout));
    //toolbar.createButton(cmd);
    helpAbout = new CommandHandler(cmd) {
        public Memento execute() {showHelpAbout(); return null;}
    };

    menu.createMenuItem(cmd=new Command(Resources.HlpContents));
    //toolbar.createButton(cmd);
    helpContents = new CommandHandler(cmd) {
        public Memento execute() {showHelpContents(); return null;}
        public boolean isInvokable() { return false; }
    };

    updateInvokers();
}
void updateInvokers() {
    jarNew.updateInvokers();
    jarOpen.updateInvokers();
    jarSave.updateInvokers();
    jarSaveAs.updateInvokers();
    jarExit.updateInvokers();
    itemAdd.updateInvokers();
    itemDelete.updateInvokers();
    itemExtract.updateInvokers();
    itemView.updateInvokers();
    itemLaunch.updateInvokers();
    helpAbout.updateInvokers();
    helpContents.updateInvokers();
}
void updateSelectionWatchers() {
    itemDelete.updateInvokers();
    itemExtract.updateInvokers();
    itemView.updateInvokers();
    itemLaunch.updateInvokers();
}

public void update(Observable o, Object arg) {
    updateInvokers();
}

//////////////////////COMMANDS////////////////////////
CommandHandler jarNew,jarOpen,jarSave,jarSaveAs,jarExit;
CommandHandler itemAdd,itemDelete,itemExtract,itemView,itemLaunch;
CommandHandler sortByName,sortByDate,sortBySize,sortByCSize;
CommandHandler helpAbout, helpContents;

void newJar() {
    if (!checkSave())
        return;
    openJar("");
}
void openJar() {
    if (!checkSave())  return;
    File file = queryArchiveFileName(FileDialog.LOAD);
    if (file == null)  return;
    if (file.exists())
        openJar(file.getPath());
    else
        WarnDialog.tell("File '"+file.getName()+"' doesn't exist!");
}
void saveJar() {
    if (checkJarName()) {
        jar.setCompression(props.getCompress());
        jar.write();
    } else
        saveJarAs();
    updateInvokers();
}
void saveJarAs() {
    File file = queryArchiveFileName(FileSelector.SAVE);
    if ((file == null)
    || (file.exists() && !WarnDialog.askOkay("'"+file.getName()+
                               "' already exists.  Overwrite?")))
        return;
    notify("renaming jar to "+file.getPath());
    jar.setName(file.getPath());
    saveJar();
}
// return true if jar has a valid name (legitimate file name), false otoh.
private boolean checkJarName() {
    // would like to use File.canWrite() but that seems to be buggy
    // (on my win95, it always returns false)
    // return (new File(jar.getName())).canWrite();

    String name = jar.getName();
    return (name!=null)
        && (name.length()>0)
        && (name.indexOf('*')==-1)
        && (name.indexOf('?')==-1);
}

// returns true if ok to continue; false if should cancel.
private boolean checkSave() {
    if (jar!=null && jar.isModified()) {
        int action = SaveDiscardCancelDialog.ask("Jar has changed.  Save?");
        if (action == SaveDiscardCancelDialog.SAVE)
            saveJar();
        else if (action == SaveDiscardCancelDialog.CANCEL)
            return false;
    }
    return true;
}
private File queryArchiveFileName(int mode) {
    String title = (mode==FileSelector.SAVE)? "Save Jar"
                 : (mode==FileSelector.LOAD)? "Open Jar"
                 : "Select file";
    FileSelector picker = new FileSelector(this, title, mode);
    String[] archiveFilters = {"*.jar", "*.zip"};
    picker.addFilenameFilter("Archives     (*.jar,*.zip)" , new FileTypeFilter(archiveFilters));
    picker.addFilenameFilter("All files    (*.*)"         , new FileTypeFilter("*.*"));
    picker.execute();
    if (!picker.wasAccepted())
        return null;
    return new File(picker.getDirectory(), picker.getFile());
}
private File queryDirectory(String title) {
    FileSelector picker = new FileSelector(this,
                                           title,
                                           FileSelector.LOAD);
    picker.addFilenameFilter("Directories only",
                             new FilenameFilter() {
        public boolean accept(File dir, String name) {
            return new File(dir, name).isDirectory();
    }   });
    picker.execute();
    if (!picker.wasAccepted())
        return null;
    // consider allowing user to highlight a dir and hit enter, selecting
    // that highlighted dir.  This implementation ignores it, and returns
    // the picker's current working directory.
    return new File(picker.getDirectory());
}

private String queryBaseDirResponse = "";
private File queryBaseDir(File base) {
    Image icon = images.ImageLoader.loadImage("question.gif");
    String OkayMsg = "Okay";
    String ChangeMsg = "Change";
    String msg = "Cannery will include directory information"
               + "\nrelative to the following base directory:"
               + "\n"
               + "      " + base.getAbsolutePath() + "  "
               + "\n";
    MessageBox box = new MessageBox();
    box.setTitle("Select Base directory");
    box.useImageCanvas(new ImageCanvas(icon));
    box.addChoice(OkayMsg);
    box.addChoice(ChangeMsg);
    box.setActionListener(new ActionListener() {
        public synchronized void actionPerformed(ActionEvent evt)
            { queryBaseDirResponse = evt.getActionCommand(); }
    });

    box.ask(msg);

    if (ChangeMsg.equals(queryBaseDirResponse)) {
        File newbase = queryDirectory("Select Base directory");
        if (newbase!=null)
            base = newbase;
    }
    return base;
}

void addItem() {
    FileSelector picker = new
    FileSelector(this, "Select file(s) to add", FileSelector.LOAD);
    String[] archiveFilters = {"*.jar", "*.zip"};
    String[] htmlFilters = {"*.htm","*.html"};
    String[] imgFilters = {"*.gif","*.jpg"};
    picker.addFilenameFilter("Class files  (*.class)"     , new FileTypeFilter("*.class"  ));
    picker.addFilenameFilter("Image files  (*.gif,*.jpg)" , new FileTypeFilter(imgFilters ));
    picker.addFilenameFilter("HTML files   (*.htm,*.html)", new FileTypeFilter(htmlFilters));
    picker.addFilenameFilter("Source files (*.java)"      , new FileTypeFilter("*.java"   ));
    picker.addFilenameFilter("Archives     (*.jar,*.zip)" , new FileTypeFilter(archiveFilters));
    picker.addFilenameFilter("All files    (*.*)"         , new FileTypeFilter("*.*"));

    picker.execute();

    if (picker.wasAccepted()) {
        File[] selected = (File[]) picker.getSelectedObjects();
        addFilesWithClosure(selected);
    }
}
private void addFilesWithClosure(File[] selected)
{
    jar.setIncludeSHADigest(props.shouldUseSHADigest());
    jar.setIncludeMD5Digest(props.shouldUseMD5Digest());

    //
    // break selected files into two sets:  classes, and non-class
    // (resource, etc) files.  Classes are put into the closure,
    // which loads dependent classes; non-classes, if any, we have
    // to figure out a relative path to know how much dir info to
    // save.
    //
    Vector nonClassFiles = new Vector();

    //
    // create and initialize the closure object from properties
    //
    JavaClassClosure closure = new JavaClassClosure();
    for (Enumeration e=props.excluded(); e.hasMoreElements(); )
        closure.excludeRoot((String) e.nextElement());

    //
    // split the passed file array into classes/non-classes.
    //
    for (int i=0; selected!=null && i<selected.length; i++) {
        File file = selected[i];

        // class files get put in the closure, which will cause them
        // to be scanned for referenced classes.
        if (file.getName().endsWith(".class")) {
            notify("scanning class " + file.getPath());

            // need to request class by name, to get the cached
            // version if available; but don't know the name until
            // after it's loaded.  So, we'll create a new copy from
            // the file, look at it's name, then throw it away and
            // use the name to get the class the right way.
            JavaClass jc = JavaClass.createFrom(file);
            JavaClass cls = JavaClass.forName(jc.getQualifiedClassName());

            if (cls!=null)
                closure.add(cls);
            else
                error("unable to load class from file '"+file.getPath()+"'");
        } else {
            // not a class; hold onto the file to load later
            nonClassFiles.addElement(file);
        }
    }

    //
    // all user-selected classes have been put into the closure, so
    // now pull them back out, along with all the classes that they
    // reference, and put them all into the jar.
    //
    boolean addSources = props.shouldAddSources();

    for (Enumeration e=closure.elements(); e.hasMoreElements(); ) {

        JavaClass cls = (JavaClass) e.nextElement();
        notify("adding class '"+cls.getQualifiedClassName()+"'");
        jar.addResource(new Resource(cls));

        if (addSources) {
            String srcName = cls.getSourceName();
            if (srcName != null) {
                notify("adding source '"+srcName+"'");
                byte[] data = JavaClass.getEnvironment().getFileBytes(srcName);
                if (data != null) {
                    long modTime = JavaClass.getEnvironment().getLastModified(srcName);
                    jar.addResource(new Resource(srcName, data, modTime));
                } else
                    Msg.error("couldn't get source file '" + srcName + "'");
            } else
                Msg.error("couldn't get source file for class "+cls.getQualifiedClassName());
        }
    }

    //
    // if there are non-class files to load, figure out the base
    // dir, then load'em.
    //
    if (nonClassFiles.size() > 0) {
        //
        //work out a string describing the base directory
        //
        File basedir = new File(System.getProperty("user.dir"));
        String base = queryBaseDir(basedir).getPath();
        base = base.replace(File.separatorChar, '/');
        if (!base.endsWith("/"))
            base += "/";

        //
        // create resources for the files, and adjust their names
        // to be relative to the selected base, then save the
        // resources into the jar.
        //
        for (int i=0; i<nonClassFiles.size(); i++) {
            File file = (File) nonClassFiles.elementAt(i);

            try {
                Resource res = new Resource(file);
                String resname = res.getName();
                if (!resname.startsWith(base))
                    throw new Error(base+" is not a parent of "+resname);

                String relpath = resname.substring(base.length());
                res.setName(relpath);

                notify("...adding item '"+file.getPath()+"'!");
                jar.addResource(res);
            }
            catch (Throwable e) {
                error(e+" loading file '"+file.getPath()+"'");
            }
        }
    }
    jar.notifyObservers();
}


void deleteItem() {
    for (Enumeration e=getSelected(); e.hasMoreElements(); ) {
        String s = (String) e.nextElement();
        notify("deleting item '"+s+"'!");
        jar.removeResource(s);
    }
    jar.notifyObservers();
}
void extractItem() {
    File dest = queryDirectory("Select target directory");
    if (dest==null || !dest.exists() || !dest.isDirectory())
        return;
    ResourceTree base = ResourceTree.createFrom(dest.getPath());

    Stack reversed = new Stack();
    for (Enumeration e=getSelected(); e.hasMoreElements(); )
        reversed.push(e.nextElement());
    Stack unscanned = new Stack();
    while (!reversed.empty())
        unscanned.push(reversed.pop());

    // unscanned holds the list of jar entries that need to be extracted.
    // An entry may be a resource(file), in which case it is copied to
    // the local file system, or it may be a subdirectory, in which case
    // it is listed, and its contents pushed onto the unscanned stack.

    // Processing continues until the unscanned stack is empty.
    while (!unscanned.empty()) {
        String entry = (String) unscanned.pop();
        if (jar.isDirectory(entry)) {
            if (base.contains(entry) && !base.isDirectory(entry))
                throw new Error("ERROR!  Can't create directory '"+entry
                    +"'; it already exists but is not a directory!");

            // Push the contents (in the jar) of the directory
            // onto the unscanned stack.  (push in reversed order, so
            // they'll come off in listed order: no reason, really)
            String[] list = jar.getDirContents(entry);
            for (int i=list.length-1; i>=0; --i)
                unscanned.push(entry + list[i]);
        } else {
            notify("extracting item '"+entry+"'!");
            Resource res = jar.getResource(entry);
            if (res != null)
                base.addResource(res);
            else Msg.error("Couldn't gt resource '"+entry+"'!");
        }
    }
    base.notifyObservers();
}

void viewItem() {}
void launchItem() {}

void showHelpAbout() {
    HelpAboutDialog.showDialog(this, "About Cannery", false);
}
void showHelpContents() {}

boolean isSelection()
    { return (jar != null && selection!=null && selection.length>0); }

Enumeration getSelected() {
    return new Enumeration() {
        int index;
        public boolean hasMoreElements()
            { return selection != null && index < selection.length; }
        public Object nextElement()
            { return selection[index++]; }
    };
}


private void openJar(String path) {
    if (path==null || path.length() == 0)
        path = System.getProperty("user.dir") + File.separator + "*.jar";

    jar = new JarArchive(new File(path));

    jar.addObserver((Observer)view);
    jar.addObserver((Observer)this);

    jar.notifyObservers();
    updateInvokers();
}



static public Color  debugTextColor = starlight.Colors.atIntensity(Color.green, 0.3);
static public Color notifyTextColor = Color.black;
static public Color   warnTextColor = starlight.Colors.atIntensity(Color.blue, 0.3);
static public Color  errorTextColor = starlight.Colors.atIntensity(Color.red, 0.3);
static public Color   puntTextColor = starlight.Colors.atIntensity(Color.red, 0.2);

public void debug(String s) {
    statusbar.setForeground(debugTextColor);
    statusbar.setText("  DEBUG: "+s);
}
public void notify(String s) {
    statusbar.setForeground(notifyTextColor);
    statusbar.setText("  "+s);
}
public void   warn(String s) {
    statusbar.setForeground(warnTextColor);
    statusbar.setText("  Warning: "+s);
}
public void  error(String s) {
    statusbar.setForeground(errorTextColor);
    statusbar.setText("  ERROR: "+s);
}
public void punt(String s) {
    statusbar.setForeground(puntTextColor);
    statusbar.setText("  FATAL: "+s);
    System.exit(1);
}

/////////////////\\\\\\\\\\\\\\\\\
boolean parseArgs(String[] args) {
    if (args.length > 0) {
        destFileName = args[0];
        // test destFileName for valid zip file name
        if (destFileName.endsWith(".zip")
         || destFileName.endsWith(".jar"))
            {}
        else
        if (destFileName.endsWith(".class")
         || destFileName.endsWith(".java")
         || destFileName.endsWith(".html")
         || destFileName.endsWith(".htm")
         || destFileName.endsWith(".gif")
         || destFileName.endsWith(".jpg"))
        {   notify("Invalid jar file type: " + destFileName);
            destFileName = "";
            return false;
        }
    } return true;
}

public Dimension getPreferredSize() { return new Dimension(380, 480); }

private void centerWindow(Window win) {
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension winDim = win.getSize();
    // If larger than screen, reduce window width or height
    if (screenDim.width < winDim.width)
        win.setSize(screenDim.width, winDim.height);
    if (screenDim.height < winDim.height)
        win.setSize(winDim.width, screenDim.height);
    int x = (screenDim.width -winDim.width )/2;
    int y = (screenDim.height-winDim.height)/2;
    win.setLocation(x, y);
}

public static void main(String args[]) {
    MainFrame frame = new MainFrame(title + " v. " + version);

    HelpAboutDialog.showTimedDialog(frame, "About Cannery", false, 5);

    MemoryMonitorWindow mem = new MemoryMonitorWindow(frame);
    BusyMonitorWindow busy = new BusyMonitorWindow(frame);

    if (frame.parseArgs(args)) {
        if (frame.destFileName!=null)
             frame.openJar(frame.destFileName);
    }
}
}
