//////////////////////license & copyright header///////////////////////
//                                                                   //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:  Kevin Kelley  //
// (kelley@iguana.ruralnet.net).                                     //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.io;

import java.io.*;

/**
*   Reverses the effects of ContinuatorOutputStream by watching
* for 'continuations' (single space at start of line) and removing
* them.
*/
public class ContinuatorInputStream extends FilterInputStream {

    static final int NORMAL=0, CRLF_STACKED=1, LF_STACKED=2, CR_STACKED=3;

    int peek  = -1;  // next char in stream, or next char after cr/lf
    int state = NORMAL;

    public ContinuatorInputStream(InputStream in) { super(in); }

    public synchronized int read() throws IOException
    {
        int ch = peek;
        if (ch == -1)
            ch = in.read();
        else peek = -1;

        switch (state) {
            case LF_STACKED: {
                state=NORMAL;
                return '\n';
            }
            case NORMAL: {
                switch (ch) {
                    case '\r': {
                        if ((ch=in.read()) == ' ')
                            return in.read();  // found continuator:  skip.
                        else if (ch == '\n') {
                            if ((ch=in.read()) == ' ')
                                return in.read();  // found continuator:  skip.
                            state = LF_STACKED;
                        }
                        peek = ch;
                        return '\r';
                    }
                    case '\n': {
                        if ((ch=in.read()) == ' ')
                            return in.read();  // found continuator:  skip.
                        peek = ch;
                    }
                    default: return ch;
                }
            }
        }
        return ch;
    }
}
