//////////////////////license & copyright header///////////////////////
//                                                                   //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:  Kevin Kelley  //
// (kelley@iguana.ruralnet.net).                                     //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.io;

import java.io.*;

/** 
*   Filters an output stream to continue lines > MaxLine bytes long,
*  as required by Manifest file format definition (continued lines 
*  begin with a single space character).
*
*   Usage:
*
*    PrintWriter print = new PrintWriter(
*                        new ContinuatorOutputStream( 
*                        new FileOutputStream("test.out")));
*/
public class ContinuatorOutputStream extends FilterOutputStream {

    /** index of next byte to be written; or, length of current line */
    int charPos;
    int MaxLineLen;

    public ContinuatorOutputStream(OutputStream out) { 
       this(out, 72);
    }
    
    public ContinuatorOutputStream(OutputStream out, int breakAfter) {
        super(out); 
        MaxLineLen = breakAfter;
    }
    
    public synchronized void write(int b) throws IOException 
    {
        charPos++;  // adjust line length
        if (b == '\r') charPos = 0;
        if (b == '\n') charPos = 0;

        if (charPos > MaxLineLen) {
            out.write('\n');
            out.write(' ');
            charPos = 2;
        }
        out.write(b);
    }

    public synchronized void write(byte b[], int off, int len) throws IOException
    {
        for (int i=off; i<off+len; i++) 
            write(b[i]);
    }

    public void flush() throws IOException {
        out.flush();
    }
}
