//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.java;
import java.io.*;

public class ConstantInfo {
static String tagTypes[] = {
"CONST_EMPTY",
"CONST_Utf8",
"invalid tag",
"CONST_Integer",
"CONST_Float",
"CONST_Long",
"CONST_Double",
"CONST_Class",
"CONST_String",
"CONST_Fieldref",
"CONST_Methodref",
"CONST_IntfMethodref",
"CONST_NameAndType"
};
public static final byte CONST_EMPTY          = 0;
public static final byte CONST_Utf8           = 1;
public static final byte CONST_Integer        = 3;
public static final byte CONST_Float          = 4;
public static final byte CONST_Long           = 5;
public static final byte CONST_Double         = 6;
public static final byte CONST_Class          = 7;
public static final byte CONST_String         = 8;
public static final byte CONST_Fieldref       = 9;
public static final byte CONST_Methodref      = 10;
public static final byte CONST_IntfMethodref  = 11;
public static final byte CONST_NameAndType    = 12;

    byte tag;
    JavaClass owningClass;

    // chase any pointers; make object self-contained
    void resolve() {}

    public Object getValue() { return null; }

    public String toString() { return tagTypes[tag]; }

    static ConstantInfo createFrom(DataInputStream in, JavaClass cls) {
        ConstantInfo c = null;
        try {
            byte tag = in.readByte();
            switch(tag) {
              case CONST_EMPTY        : break;
              case CONST_Utf8         : c = CONSTANT_Utf8         .createFrom(in); break;
              case CONST_Integer      : c = CONSTANT_Integer      .createFrom(in); break;
              case CONST_Float        : c = CONSTANT_Float        .createFrom(in); break;
              case CONST_Long         : c = CONSTANT_Long         .createFrom(in); break;
              case CONST_Double       : c = CONSTANT_Double       .createFrom(in); break;
              case CONST_String       : c = CONSTANT_String       .createFrom(in); break;
              case CONST_Class        : c = CONSTANT_Class        .createFrom(in); break;
              case CONST_Fieldref     : c = CONSTANT_Fieldref     .createFrom(in); break;
              case CONST_Methodref    : c = CONSTANT_Methodref    .createFrom(in); break;
              case CONST_IntfMethodref: c = CONSTANT_IntfMethodref.createFrom(in); break;
              case CONST_NameAndType  : c = CONSTANT_NameAndType  .createFrom(in); break;
              default: throw new Error("Bad tag <"+tag+"> in ConstantInfo!");
            }
            if (c != null) {
                c.tag = tag;
                c.owningClass = cls;
            }
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {}
}

class CONSTANT_Utf8 extends ConstantInfo {  
    String string;

    public Object getValue() { return string; }
    public void setValue(Object o) {
        if (!(o instanceof String)) 
            throw new Error("Invalid assignment:  need a String.");
        string = (String) o;
    }
    public String toString() { return super.toString() + " '" + string + "'"; }

    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_Utf8 c = new CONSTANT_Utf8();
        try {
            c.string = in.readUTF();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            out.writeUTF(string);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
}

class CONSTANT_Integer extends ConstantInfo {  
        int  value;  
    public Object getValue() { return new Integer(value); }
    public void setValue(Object o) {
        if (!(o instanceof Integer)) 
            throw new Error("Invalid assignment:  need an Integer.");
        value = ((Integer)o).intValue();
    }
    public String toString() { return super.toString() + " <" + String.valueOf(value) + ">"; }

    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_Integer c = new CONSTANT_Integer();
        try {
            c.value = in.readInt();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            out.writeInt(value);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
} 

class CONSTANT_Float extends ConstantInfo {  
        float  value;   // IEEE rep
    public Object getValue() { return new Float(value); }
    public void setValue(Object o) {
        if (!(o instanceof Float)) 
            throw new Error("Invalid assignment:  need a Float.");
        value = ((Float)o).floatValue();
    }
    public String toString() { return super.toString() + " <" + String.valueOf(value) + "F>"; }

    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_Float c = new CONSTANT_Float();
        try {
            c.value = in.readFloat();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            out.writeFloat(value);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
} 

class CONSTANT_Long extends ConstantInfo {       // takes two constant_pool slots
        long  value;
    public Object getValue() { return new Long(value); }
    public void setValue(Object o) {
        if (!(o instanceof Long)) 
            throw new Error("Invalid assignment:  need a Long.");
        value = ((Long)o).longValue();
    }
    public String toString() { return super.toString() + " <" + String.valueOf(value) + "L>"; }

    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_Long c = new CONSTANT_Long();
        try {
            c.value = in.readLong();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            out.writeLong(value);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
} 

class CONSTANT_Double extends ConstantInfo {     // takes two constant_pool slots
        double  value;
    public Object getValue() { return new Double(value); }
    public void setValue(Object o) {
        if (!(o instanceof Double)) 
            throw new Error("Invalid assignment:  need a Double.");
        value = ((Double)o).doubleValue();
    }
    public String toString() { return super.toString() + " <" + String.valueOf(value) + "D>"; }

    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_Double c = new CONSTANT_Double();
        try {
            c.value = in.readDouble();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            out.writeDouble(value);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
}  

class CONSTANT_String extends ConstantInfo {  
    int            stringIndex;
    CONSTANT_Utf8  string;

    void resolve() {
        if (string == null) { 
            string = (CONSTANT_Utf8) owningClass.constantAtIndex(stringIndex);
            stringIndex = -1;
        }
    }
    public Object getValue()         { resolve(); return string.getValue(); }
    public void   setValue(Object o) { resolve(); string.setValue(o); }

    public String toString() { 
        resolve(); 
        return super.toString() + " \"" + string.string + '"'; 
    }

    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_String c = new CONSTANT_String();
        try {
            c.stringIndex = in.readUnsignedShort();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            if (stringIndex == -1) 
                stringIndex = owningClass.indexOf(string);
            out.writeShort(stringIndex);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
}  


class CONSTANT_Class extends ConstantInfo {  
    int     nameIndex;     // Because arrays are objects, the opcodes 
                            // anewarray and multianewarray can reference
                            // array "classes" via CONSTANT_Class items 
                            // in the constant pool. In this case, the 
                            // name of the class is its signature. For 
                            // example, the class name for  
                            //        int[][]    is  [[I  
                            //        Thread[]   is  "[Ljava.lang.Thread;"  
    CONSTANT_Utf8  className;

    void resolve() {
        if (className == null) { 
            className = (CONSTANT_Utf8) owningClass.constantAtIndex(nameIndex);
            nameIndex = -1;
        }
    }
    public String getClassName() { 
        resolve(); 
        String internalFormat = (String) className.getValue();
        return internalFormat.replace('/', '.'); 
    }
    public void setClassName(String n) {
        resolve(); 
        String internalFormat = n.replace('.', '/');
        className.setValue(n); 
    }

    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_Class c = new CONSTANT_Class();
        try {
            c.nameIndex = in.readUnsignedShort();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            if (nameIndex == -1) 
                nameIndex = owningClass.indexOf(className);
            out.writeShort(nameIndex);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
    public String toString() { return super.toString() + " " + getClassName() + ""; }
}

class ConstantDualReference extends ConstantInfo {
    int     classIndex;       // CONSTANT_Class  
    int     nameAndTypeIndex; // CONSTANT_NameAndType. 

    CONSTANT_Class        className;
    CONSTANT_NameAndType  nameAndType;

    void resolve() {
        if (className == null) { 
            className   = (CONSTANT_Class)       owningClass.constantAtIndex(classIndex);
            nameAndType = (CONSTANT_NameAndType) owningClass.constantAtIndex(nameAndTypeIndex);
            classIndex       = -1;
            nameAndTypeIndex = -1;
        }
    }
    public Object getClassName()        { resolve(); return className.getClassName(); }
    public void  setClassName(String n) { resolve(); className.setClassName(n); }

    public Object getName()        { resolve(); return nameAndType.getName(); }
    public void  setName(String n) { resolve(); nameAndType.setName(n); }

    public Object getDescriptor()        { resolve(); return nameAndType.getDescriptor(); }
    public void  setDescriptor(String n) { resolve(); nameAndType.setDescriptor(n); }


    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            if (classIndex == -1) 
                classIndex = owningClass.indexOf(className);
            out.writeShort(classIndex);
            if (nameAndTypeIndex == -1) 
                nameAndTypeIndex = owningClass.indexOf(nameAndType);
            out.writeShort(nameAndTypeIndex);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
    public String toString() { 
        resolve(); 
        return super.toString() 
        + " " + className.getClassName()
        + "::" + nameAndType.getName()
        + " <" + nameAndType.getDescriptor() + ">";
    }
}

class CONSTANT_Fieldref extends ConstantDualReference {  
    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_Fieldref c = new CONSTANT_Fieldref();
        try {
            c.classIndex = in.readUnsignedShort();
            c.nameAndTypeIndex = in.readUnsignedShort();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
} 
class CONSTANT_Methodref extends ConstantDualReference {  
    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_Methodref c = new CONSTANT_Methodref();
        try {
            c.classIndex = in.readUnsignedShort();
            c.nameAndTypeIndex = in.readUnsignedShort();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
}  
class CONSTANT_IntfMethodref extends ConstantDualReference {  
    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_IntfMethodref c = new CONSTANT_IntfMethodref();
        try {
            c.classIndex = in.readUnsignedShort();
            c.nameAndTypeIndex = in.readUnsignedShort();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
}  

class CONSTANT_NameAndType extends ConstantInfo {  

    int     nameIndex;       // utf8 field/method name
    int     sigIndex;        // utf8 field/method descriptor

    CONSTANT_Utf8 name;
    CONSTANT_Utf8 descriptor;

    void resolve() {
        if (name == null) { 
            name       = (CONSTANT_Utf8) owningClass.constantAtIndex(nameIndex);
            descriptor = (CONSTANT_Utf8) owningClass.constantAtIndex(sigIndex);
            nameIndex = -1;
            sigIndex  = -1;
        }
    }
    public String getName()         { resolve(); return (String) name.getValue(); }
    public void   setName(String n) { resolve(); name.setValue(n); }

    public String getDescriptor()         { resolve(); return (String) descriptor.getValue(); }
    public void   setDescriptor(String n) { resolve(); descriptor.setValue(n); }

    static ConstantInfo createFrom(DataInputStream in) {
        CONSTANT_NameAndType c = new CONSTANT_NameAndType();
        try {
            c.nameIndex = in.readUnsignedShort();
            c.sigIndex = in.readUnsignedShort();
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeByte(tag);
            if (nameIndex == -1) 
                nameIndex = owningClass.indexOf(name);
            out.writeShort(nameIndex);
            if (sigIndex == -1) 
                sigIndex = owningClass.indexOf(descriptor);
            out.writeShort(sigIndex);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
    public String toString() { 
        return super.toString() 
        + " " + getName()
        + " <" + getDescriptor() + ">";
    }
}  
