//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.java;
import java.io.*;
import java.util.*;
import starlight.util.Msg;

public class Directory extends ResourceTree {

    private File path;
    public Directory(File f) {
        path = f;
        separator     = File.separator;
        separatorChar = File.separatorChar;
    }


    public String getName() { return path.getAbsolutePath(); }

    public void setName(String newname)
        { if (!path.renameTo(new File(newname))) Msg.error("rename failed"); }

    public String[] getDirContents(String subdir) {
        return getDirContents(subdir, null);
    }
    public String[] getDirContents(String subdir, FilenameFilter filt) {
        File dir = fileForName(subdir);
        String[] list = (filt==null)? dir.list(): dir.list(filt);
        if (list == null) list = new String[0];
        for (int i=0; i<list.length; i++) {
            File f = new File(dir, list[i]);
            if (f.isDirectory() && !list[i].endsWith(separator))
                list[i] += separator;
        }
        return list;
    }

    public Enumeration childNames() {
        return new Enumeration() {
            Stack unscanned = new Stack();

            // seed the stack with the 'root' contents
            boolean placeholder = pushSubdirs(path);

            private boolean pushSubdirs(File dir) {
                String[] contents = dir.list();
                for (int i=contents.length-1; i>=0; --i)
                    unscanned.push(contents[i]);
                return false;
            }
            public boolean hasMoreElements() { return !unscanned.empty(); }

            public Object nextElement() {
                String next = (String) unscanned.pop();
                File file = fileForName(next);
                if (file.isDirectory()) {
                    next += separator;
                    String[] contents = file.list();
                    for (int i=contents.length-1; i>=0; --i)
                        unscanned.push(next + contents[i]);
                } return next;
            }
        };
    }


    public boolean isDirectory(String qualName) {
        return fileForName(qualName).isDirectory();
    }
    public boolean contains(String qualName) {
        try { return fileForName(qualName).exists();
        } catch (Exception e) {}
        return false;
    }
    public long lastModified(String qualName) {
        long mod = -1;
        try { mod = fileForName(qualName).lastModified();
        } catch (Exception e) {}
        return mod;
    }
    public byte[] getBytes(String qualName) {
        byte[] data = null;
        try{ InputStream in = new FileInputStream(fileForName(qualName));
            data = new byte[in.available()];
            in.read(data);
            in.close();
        } catch (IOException e) {}
        return data;
    }
    public Resource getResource(String qualName) {
        try {
            return new Resource(fileForName(qualName));
        } catch(Exception e) { return null; }
    }

    public void addResource(Resource res) {
        File dest = fileForName(res.getName());
        File parent = new File(dest.getParent());
        if (dest.exists())
            Msg.debug("resource "+dest.getName()+" already exists!");
        if (!parent.exists())
            parent.mkdirs();
        OutputStream out = getOutputStream(res.getName());
        if (out != null) try {
            out.write(res.getContent());
        } catch(Exception e) {
            Msg.error(e + " saving resource");
        } finally {
            try { out.close(); } catch(IOException e) {}
        }
    }

    public void removeResource(String qualName) {
        File file = fileForName(qualName);
        if (!file.exists()) throw new Error("Can't remove nonexistent file!");
        file.delete();
    }


    // create an output stream suitable for updating a child resource
    protected OutputStream getOutputStream(String qualName) {
        try {
            return new BufferedOutputStream(
                   new FileOutputStream( fileForName(qualName) ));
        } catch (IOException e) { Msg.error("getting output stream"); }
        return null;
    }

    /////////////////////////////////
    File fileForName(String qualName) {
        String locallized = qualName.replace('/', separatorChar);
        return new File(path, locallized);
    }
}
