//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.java;
import java.util.Hashtable;
import java.util.Enumeration;

public class DirectoryTree {

    class Node {
        String name;
        Object value;
        public Node(String n) {name=n;}
        public Node(String n, Object val) {this(n); value=val;}
        Node get(String name) {return null;}
        Node put(String name, Node node) {return null;}
        void remove(String name) {}
        String[] list() {return null; }
    }
    class DirNode extends Node {
        Hashtable children = new Hashtable();
        DirNode(String n) {super(n);}
        Node get(String name) { return (Node) children.get(name); }
        Node put(String name, Node node) { children.put(name, node); return node; }
        void remove(String name) { children.remove(name); }
        String[] list() {
            String[] elts = new String[children.size()];
            int index = 0;
            for (Enumeration e=children.keys(); e.hasMoreElements(); )
                elts[index++] = (String) e.nextElement();
            return elts;
        }
    }

    Node base;
    char separatorChar = '/';

    public DirectoryTree(String name) { base = new DirNode(name); }
    public DirectoryTree(String name, char sep) { this(name); separatorChar = sep; }

    public void createFrom(String name, char sep, Enumeration contentNames) {
        separatorChar = sep;
        base = new DirNode(name);           // discard the old tree
        while (contentNames.hasMoreElements())
            add((String) contentNames.nextElement());
    }

    Node add(String qualName, Object res) {
        Node node = add(qualName);
        node.value = res;
        return node;
    }
    Node add(String qualname) {
        int ix = qualname.lastIndexOf(separatorChar);
        String path = (ix >= 0) ? qualname.substring(0, ix+1): "";
        String name = (ix >= 0) ? qualname.substring(ix+1): qualname;
        Node node = base;
        if (path.length() > 0) node = force(path);
        if (name.length() > 0) node = node.put(name, new Node(name));
        return node;
    }
    void remove(String qualname) {
        int ix = qualname.lastIndexOf(separatorChar, qualname.length()-2);
        String path = (ix >= 0) ? qualname.substring(0, ix+1): "";
        String name = (ix >= 0) ? qualname.substring(ix+1): qualname;
        Node node = base;
        if (path.length() > 0) node = get(path);
        if (name.length() > 0) node.remove(name);
    }
    Node force(String path) {
        Node node = base;
        while (path != null && path.length() > 0) {
            String dir = path;
            int ix = path.indexOf(separatorChar);
            if (ix > 0) {
                dir  = path.substring(0, ix+1);
                path = path.substring(ix+1);
            } else path = null;
            Node ch = node.get(dir);
            if (ch == null) {
                ch = new DirNode(dir);
                node.put(dir, ch);
            }
            node = ch;
        }
        return node;
    }
    Node get(String path) {
        Node node = base;
        while (node != null && path != null && path.length() > 0) {
            String dir = path;
            int ix = path.indexOf('/');
            if (ix > 0) {
                dir  = path.substring(0, ix+1);
                path = path.substring(ix+1);
            } else path = null;
            node = node.get(dir);
        }
        return node;
    }
    String[] list(String dirname) {
        Node node = get(dirname);
        if (!(node instanceof DirNode))
            throw new Error("can't list non-directory " +dirname);
        return node.list();
    }
}
