//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.java;
import java.io.*;

public class DisAssembler {

byte[] code;
DataInputStream in;

public DisAssembler(byte[] code) { this.code = code; reset(); }

public void reset()     {   in = new DataInputStream
                                (new ByteArrayInputStream(code));
                        }
public int at()         { try {
                            return code.length - in.available(); 
                          } catch (Exception e) { throw new Error(e.toString()); }
                        }
public void to(int mark){ try {
                            if (at() <= mark) in.skipBytes(mark-at());
                            else { reset(); in.skipBytes(mark); }
                          } catch (Exception e) { throw new Error(e.toString()); }
                        }
public boolean eoi()    { try {
                            return in.available() <= 0; 
                          } catch (Exception e) { throw new Error(e.toString()); }
                        }
public int available()  { try {
                            return in.available(); 
                          } catch (Exception e) { throw new Error(e.toString()); }
                        }

void alignBy4()         { try {
                            int align = (at() + 3) & ~3;
                            in.skipBytes(align - at());
                          } catch (Exception e) { throw new Error(e.toString()); }
                        }

public String readInstruction() 
{
  int opcode=0;
  try {
    opcode = in.readUnsignedByte();
    switch (opcode) {
    case   0 /*0x00*/ : return "nop";  
    case   1 /*0x01*/ : return "aconst_null";  
    case   2 /*0x02*/ : return "iconst_m1";  
    case   3 /*0x03*/ : return "iconst_0";  
    case   4 /*0x04*/ : return "iconst_1";  
    case   5 /*0x05*/ : return "iconst_2";  
    case   6 /*0x06*/ : return "iconst_3";  
    case   7 /*0x07*/ : return "iconst_4";  
    case   8 /*0x08*/ : return "iconst_5";  
    case   9 /*0x09*/ : return "lconst_0";
    case  10 /*0x0a*/ : return "lconst_1";
    case  11 /*0x0b*/ : return "fconst_0";
    case  12 /*0x0c*/ : return "fconst_1";
    case  13 /*0x0d*/ : return "fconst_2";
    case  14 /*0x0e*/ : return "dconst_0";
    case  15 /*0x0f*/ : return "dconst_1";
    case  16 /*0x10*/ : return "bipush " + in.readByte(); 
    case  17 /*0x11*/ : return "sipush " + in.readShort(); 
    case  18 /*0x12*/ : return "ldc *" + in.readUnsignedByte();   
    case  19 /*0x13*/ : return "ldc_w *" + in.readUnsignedShort();
    case  20 /*0x14*/ : return "ldc2_w *" + in.readUnsignedShort();  
    case  21 /*0x15*/ : return "iload #" + in.readUnsignedByte();  
    case  22 /*0x16*/ : return "lload #" + in.readUnsignedByte();  
    case  23 /*0x17*/ : return "fload #" + in.readUnsignedByte();    
    case  24 /*0x18*/ : return "dload #" + in.readUnsignedByte();    
    case  25 /*0x19*/ : return "aload #" + in.readUnsignedByte();    
    case  26 /*0x1a*/ : return "iload_0";
    case  27 /*0x1b*/ : return "iload_1";
    case  28 /*0x1c*/ : return "iload_2";
    case  29 /*0x1d*/ : return "iload_3";
    case  30 /*0x1e*/ : return "lload_0";
    case  31 /*0x1f*/ : return "lload_1";
    case  32 /*0x20*/ : return "lload_2";
    case  33 /*0x21*/ : return "lload_3";
    case  34 /*0x22*/ : return "fload_0";
    case  35 /*0x23*/ : return "fload_1";
    case  36 /*0x24*/ : return "fload_2";
    case  37 /*0x25*/ : return "fload_3";
    case  38 /*0x26*/ : return "dload_0";
    case  39 /*0x27*/ : return "dload_1";
    case  40 /*0x28*/ : return "dload_2";
    case  41 /*0x29*/ : return "dload_3";
    case  42 /*0x2a*/ : return "aload_0";
    case  43 /*0x2b*/ : return "aload_1";
    case  44 /*0x2c*/ : return "aload_2";
    case  45 /*0x2d*/ : return "aload_3";
    case  46 /*0x2e*/ : return "iaload";  
    case  47 /*0x2f*/ : return "laload";  
    case  48 /*0x30*/ : return "faload";  
    case  49 /*0x31*/ : return "daload";  
    case  50 /*0x32*/ : return "aaload";  
    case  51 /*0x33*/ : return "baload";  
    case  52 /*0x34*/ : return "caload";  
    case  53 /*0x35*/ : return "saload";  
    case  54 /*0x36*/ : return "istore #" + in.readUnsignedByte();    
    case  55 /*0x37*/ : return "lstore #" + in.readUnsignedByte();    
    case  56 /*0x38*/ : return "fstore #" + in.readUnsignedByte();    
    case  57 /*0x39*/ : return "dstore #" + in.readUnsignedByte();    
    case  58 /*0x3a*/ : return "astore #" + in.readUnsignedByte();    
    case  59 /*0x3b*/ : return "istore_0";  
    case  60 /*0x3c*/ : return "istore_1"; 
    case  61 /*0x3d*/ : return "istore_2";
    case  62 /*0x3e*/ : return "istore_3";  
    case  63 /*0x3f*/ : return "lstore_0";
    case  64 /*0x40*/ : return "lstore_1";  
    case  65 /*0x41*/ : return "lstore_2";  
    case  66 /*0x42*/ : return "lstore_3";  
    case  67 /*0x43*/ : return "fstore_0";  
    case  68 /*0x44*/ : return "fstore_1"; 
    case  69 /*0x45*/ : return "fstore_2";  
    case  70 /*0x46*/ : return "fstore_3";  
    case  71 /*0x47*/ : return "dstore_0";  
    case  72 /*0x48*/ : return "dstore_1";  
    case  73 /*0x49*/ : return "dstore_2";  
    case  74 /*0x4a*/ : return "dstore_3";  
    case  75 /*0x4b*/ : return "astore_0";  
    case  76 /*0x4c*/ : return "astore_1";  
    case  77 /*0x4d*/ : return "astore_2";  
    case  78 /*0x4e*/ : return "astore_3";  
    case  79 /*0x4f*/ : return "iastore";  
    case  80 /*0x50*/ : return "lastore";  
    case  81 /*0x51*/ : return "fastore";  
    case  82 /*0x52*/ : return "dastore";  
    case  83 /*0x53*/ : return "aastore";  
    case  84 /*0x54*/ : return "bastore";  
    case  85 /*0x55*/ : return "castore";  
    case  86 /*0x56*/ : return "sastore";  
    case  87 /*0x57*/ : return "pop";  
    case  88 /*0x58*/ : return "pop2";  
    case  89 /*0x59*/ : return "dup";  
    case  90 /*0x5a*/ : return "dup_x1";  
    case  91 /*0x5b*/ : return "dup_x2";  
    case  92 /*0x5c*/ : return "dup2";  
    case  93 /*0x5d*/ : return "dup2_x1";  
    case  94 /*0x5e*/ : return "dup2_x2";  
    case  95 /*0x5f*/ : return "swap";  
    case  96 /*0x60*/ : return "iadd";  
    case  97 /*0x61*/ : return "ladd";  
    case  98 /*0x62*/ : return "fadd";  
    case  99 /*0x63*/ : return "dadd";  
    case 100 /*0x64*/ : return "isub";  
    case 101 /*0x65*/ : return "lsub";  
    case 102 /*0x66*/ : return "fsub";  
    case 103 /*0x67*/ : return "dsub";  
    case 104 /*0x68*/ : return "imul";  
    case 105 /*0x69*/ : return "lmul";  
    case 106 /*0x6a*/ : return "fmul";  
    case 107 /*0x6b*/ : return "dmul";  
    case 108 /*0x6c*/ : return "idiv";  
    case 109 /*0x6d*/ : return "ldiv";  
    case 110 /*0x6e*/ : return "fdiv";  
    case 111 /*0x6f*/ : return "ddiv";  
    case 112 /*0x70*/ : return "irem";  
    case 113 /*0x71*/ : return "lrem";  
    case 114 /*0x72*/ : return "frem";  
    case 115 /*0x73*/ : return "drem";  
    case 116 /*0x74*/ : return "ineg";  
    case 117 /*0x75*/ : return "lneg";  
    case 118 /*0x76*/ : return "fneg";  
    case 119 /*0x77*/ : return "dneg";  
    case 120 /*0x78*/ : return "ishl";  
    case 121 /*0x79*/ : return "lshl";  
    case 122 /*0x7a*/ : return "ishr";  
    case 123 /*0x7b*/ : return "lshr";  
    case 124 /*0x7c*/ : return "iushr";  
    case 125 /*0x7d*/ : return "lushr";  
    case 126 /*0x7e*/ : return "iand";  
    case 127 /*0x7f*/ : return "land";  
    case 128 /*0x80*/ : return "ior";  
    case 129 /*0x81*/ : return "lor";  
    case 130 /*0x82*/ : return "ixor";  
    case 131 /*0x83*/ : return "lxor";  
    case 132 /*0x84*/ : return "iinc #" + in.readUnsignedByte() + " " + in.readByte();    
    case 133 /*0x85*/ : return "i2l";  
    case 134 /*0x86*/ : return "i2f";  
    case 135 /*0x87*/ : return "i2d";  
    case 136 /*0x88*/ : return "l2i";  
    case 137 /*0x89*/ : return "l2f";  
    case 138 /*0x8a*/ : return "l2d";  
    case 139 /*0x8b*/ : return "f2i";  
    case 140 /*0x8c*/ : return "f2l";  
    case 141 /*0x8d*/ : return "f2d";  
    case 142 /*0x8e*/ : return "d2i";  
    case 143 /*0x8f*/ : return "d2l";  
    case 144 /*0x90*/ : return "d2f";  
    case 145 /*0x91*/ : return "i2b";  
    case 146 /*0x92*/ : return "i2c";  
    case 147 /*0x93*/ : return "i2s";  
    case 148 /*0x94*/ : return "lcmp";  
    case 149 /*0x95*/ : return "fcmpl";  
    case 150 /*0x96*/ : return "fcmpg";  
    case 151 /*0x97*/ : return "dcmpl";  
    case 152 /*0x98*/ : return "dcmpg";  
    case 153 /*0x99*/ : return "ifeq +" + in.readShort();   
    case 154 /*0x9a*/ : return "ifne +" + in.readShort(); 
    case 155 /*0x9b*/ : return "iflt +" + in.readShort();  
    case 156 /*0x9c*/ : return "ifge +" + in.readShort();  
    case 157 /*0x9d*/ : return "ifgt +" + in.readShort();  
    case 158 /*0x9e*/ : return "ifle +" + in.readShort();  
    case 159 /*0x9f*/ : return "if_icmpeq +" + in.readShort();  
    case 160 /*0xa0*/ : return "if_icmpne +" + in.readShort();  
    case 161 /*0xa1*/ : return "if_icmplt +" + in.readShort();  
    case 162 /*0xa2*/ : return "if_icmpge +" + in.readShort();  
    case 163 /*0xa3*/ : return "if_icmpgt +" + in.readShort();  
    case 164 /*0xa4*/ : return "if_icmple +" + in.readShort(); 
    case 165 /*0xa5*/ : return "if_acmpeq +" + in.readShort();  
    case 166 /*0xa6*/ : return "if_acmpne +" + in.readShort(); 
    case 167 /*0xa7*/ : return "goto +" + in.readShort();  
    case 168 /*0xa8*/ : return "jsr +" + in.readShort();  
    case 169 /*0xa9*/ : return "ret #" + in.readUnsignedByte();  
    case 170 /*0xaa*/ : {
        alignBy4();
        int dflt = in.readInt();
        int lo = in.readInt();
        int hi = in.readInt();
        int count = hi - lo + 1;
        int[] table = new int[count];
        for (int i=0; i<count; i++) 
            table[i] = in.readInt();
        String rtn = "tableswitch  +" + dflt + " " + lo + ".." + hi + " {";
        for (int i=0; i<count; i++) {
            rtn += table[i];
            if (i < count-1) rtn += ", ";
        }
        rtn += "}";
        return rtn;  
    }
    case 171 /*0xab*/ : {
        alignBy4();
        int dflt = in.readInt();
        int npairs = in.readInt();
        int[] table = new int[npairs*2];
        for (int i=0; i<npairs*2; i++) 
            table[i] = in.readInt();
        String rtn = "lookupswitch  +" + dflt + " " + npairs + " {";
        for (int i=0; i<npairs; i++) {
            rtn += table[2*i] + ":" + table[2*i+1];
            if (i < npairs-1) rtn += "; ";
        }
        rtn += "}";
        return rtn;  
    }
    case 172 /*0xac*/ : return "ireturn";  
    case 173 /*0xad*/ : return "lreturn";  
    case 174 /*0xae*/ : return "freturn";  
    case 175 /*0xaf*/ : return "dreturn";  
    case 176 /*0xb0*/ : return "areturn";  
    case 177 /*0xb1*/ : return "return";  
    case 178 /*0xb2*/ : return "getstatic *" + in.readUnsignedShort();  
    case 179 /*0xb3*/ : return "putstatic *" + in.readUnsignedShort();  
    case 180 /*0xb4*/ : return "getfield *" + in.readUnsignedShort();  
    case 181 /*0xb5*/ : return "putfield *" + in.readUnsignedShort();  
    case 182 /*0xb6*/ : return "invokevirtual *" + in.readUnsignedShort();   
    case 183 /*0xb7*/ : return "invokespecial *" + in.readUnsignedShort();   
    case 184 /*0xb8*/ : return "invokestatic *" + in.readUnsignedShort();   
    case 185 /*0xb9*/ : return "invokeinterface *" + in.readUnsignedShort() +" "+ in.readUnsignedByte() +" "+ in.readByte();   
    case 186 /*0xba*/ : return "xxxunusedxxx";  
    case 187 /*0xbb*/ : return "new *" + in.readUnsignedShort();   
    case 188 /*0xbc*/ : return "newarray " + in.readUnsignedByte();   
    case 189 /*0xbd*/ : return "anewarray *" + in.readUnsignedShort();   
    case 190 /*0xbe*/ : return "arraylength";  
    case 191 /*0xbf*/ : return "athrow";  
    case 192 /*0xc0*/ : return "checkcast *" + in.readUnsignedShort();   
    case 193 /*0xc1*/ : return "instanceof *" + in.readUnsignedShort();   
    case 194 /*0xc2*/ : return "monitorenter";  
    case 195 /*0xc3*/ : return "monitorexit";  
    case 196 /*0xc4*/ : {
      int op = in.readUnsignedByte();
      switch (op) {
        case  21 /*0x15*/ : return "iload_wide #" + in.readUnsignedShort();  
        case  22 /*0x16*/ : return "lload_wide #" + in.readUnsignedShort();  
        case  23 /*0x17*/ : return "fload_wide #" + in.readUnsignedShort();    
        case  24 /*0x18*/ : return "dload_wide #" + in.readUnsignedShort();    
        case  25 /*0x19*/ : return "aload_wide #" + in.readUnsignedShort();    
        case  54 /*0x36*/ : return "istore_wide #" + in.readUnsignedShort();    
        case  55 /*0x37*/ : return "lstore_wide #" + in.readUnsignedShort();    
        case  56 /*0x38*/ : return "fstore_wide #" + in.readUnsignedShort();    
        case  57 /*0x39*/ : return "dstore_wide #" + in.readUnsignedShort();    
        case  58 /*0x3a*/ : return "astore_wide #" + in.readUnsignedShort();    
        case 169 /*0xa9*/ : return "ret_wide #" + in.readUnsignedShort();  
        case 132 /*0x84*/ : return "iinc_wide #" + in.readUnsignedShort() + " " + in.readShort();    
      }
      return "wide_" + op;
    }
    case 197 /*0xc5*/ : return "multianewarray *" + in.readUnsignedShort() +" "+ in.readUnsignedByte();   
    case 198 /*0xc6*/ : return "ifnull +" + in.readShort();   
    case 199 /*0xc7*/ : return "ifnonnull +" + in.readShort();   
    case 200 /*0xc8*/ : return "goto_w +" + in.readInt();
    case 201 /*0xc9*/ : return "jsr_w +" + in.readInt();  

    //'invisible' _quick opcodes:
    case 203 /*0xcb*/ : return "ldc_quick *" + in.readUnsignedByte();  
    case 204 /*0xcc*/ : return "ldc_w_quick *" + in.readUnsignedShort();  
    case 205 /*0xcd*/ : return "ldc2_w_quick *" + in.readUnsignedShort();  
    case 206 /*0xce*/ : return "getfield_quick " + in.readUnsignedByte() +" "+ in.readUnsignedByte();  
    case 207 /*0xcf*/ : return "putfield_quick " + in.readUnsignedByte() +" "+ in.readUnsignedByte();  
    case 208 /*0xd0*/ : return "getfield2_quick " + in.readUnsignedByte() +" "+ in.readUnsignedByte();  
    case 209 /*0xd1*/ : return "putfield2_quick " + in.readUnsignedByte() +" "+ in.readUnsignedByte();  
    case 210 /*0xd2*/ : return "getstatic_quick *" + in.readUnsignedShort();  
    case 211 /*0xd3*/ : return "putstatic_quick *" + in.readUnsignedShort();  
    case 212 /*0xd4*/ : return "getstatic2_quick *" + in.readUnsignedShort();  
    case 213 /*0xd5*/ : return "putstatic2_quick *" + in.readUnsignedShort();  
    case 214 /*0xd6*/ : return "invokevirtual_quick " + in.readUnsignedByte() +" "+ in.readUnsignedByte();  
    case 215 /*0xd7*/ : return "invokenonvirtual_quick *" + in.readUnsignedShort();  
    case 216 /*0xd8*/ : return "invokesuper_quick *" + in.readUnsignedShort();  
    case 217 /*0xd9*/ : return "invokestatic_quick *" + in.readUnsignedShort();  
    case 218 /*0xda*/ : return "invokeinterface_quick *" + in.readUnsignedShort() +" "+ in.readUnsignedByte() +" "+ in.readByte();  
    case 219 /*0xdb*/ : return "invokevirtualobject_quick " + in.readUnsignedByte() +" "+ in.readUnsignedByte();  
    case 221 /*0xdd*/ : return "new_quick *" + in.readUnsignedShort();  
    case 222 /*0xde*/ : return "anewarray_quick *" + in.readUnsignedShort();  
    case 223 /*0xdf*/ : return "multianewarray_quick *" + in.readUnsignedShort() +" "+ in.readUnsignedByte();  
    case 224 /*0xe0*/ : return "checkcast_quick *" + in.readUnsignedShort();  
    case 225 /*0xe1*/ : return "instanceof_quick *" + in.readUnsignedShort();  
    case 226 /*0xe2*/ : return "invokevirtual_quick_w";  
    case 227 /*0xe3*/ : return "getfield_quick_w *" + in.readUnsignedShort();  
    case 228 /*0xe4*/ : return "putfield_quick_w *" + in.readUnsignedShort();  

    //Reserved opcodes:
    case 202 /*0xca*/ : return "breakpoint";
    case 254 /*0xfe*/ : return "impdep1";
    case 255 /*0xff*/ : return "impdep2";

    default: System.err.println("invalid opcode <"+opcode+"> at offset "+at());
            return "BAD_OPCODE_" + opcode;
    }
  } catch (Exception e) { throw new Error(e.toString()); }
}
}
