//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.java;
import java.io.*;
import java.util.*;
import starlight.util.ByteArray;

public class JavaClass {

    static public  void setEnvironment(Environment env)
        { environment = env; }
    static public  Environment getEnvironment()
        { return environment; }
    static private  Environment environment
        = new JDKEnvironment();


    static Hashtable classcache = new Hashtable();

    /** does a lookup on the name according to the rules of the
      * currently set environment (by default, uses the one that
      * java uses).  Defines and returns a JavaClass object
      * representing that class.  Returns cached class, if such
      * exists; otherwise loads.
      */
    public static JavaClass forName(String qualifiedName)
        {
            if (qualifiedName.indexOf('/') >= 0)  throw new Error
                    ("internal format: '" + qualifiedName + "'");
            if (qualifiedName.indexOf('[') >= 0)
                qualifiedName = Signature.createFrom(qualifiedName).basicTypeName();

            JavaClass cls = (JavaClass) classcache.get(qualifiedName);
            if (cls==null) {
                String filename = fileNameForClassName(qualifiedName);
                byte[] data = getEnvironment().getFileBytes(filename);
                if (data == null) throw new Error
                    ("couldn't get bytes of class '" + qualifiedName + "'");
                cls = createFrom(data);
                cls.modificationTime = getEnvironment().getLastModified(filename);
                classcache.put(qualifiedName, cls);
            }
            return cls;
        }
    public static JavaClass forSignature(Signature sig)
        { return JavaClass.forName(sig.basicTypeName()); }


    ByteArrayOutputStream bs;
    public byte[] getClassBytes()
        {   if (bs==null)  bs = new ByteArrayOutputStream(2048);
            bs.reset();
            writeTo(new DataOutputStream(bs));
            return bs.toByteArray();
        }

    public AccessFlags getAccessFlags()
        { return new AccessFlags(accessFlags); }

    public boolean isClass()
        { return !getAccessFlags().isInterface(); }

    public boolean isInterface()
        { return getAccessFlags().isInterface(); }

    public boolean isJavaBean()
        { return false; }

    public static String fileNameForClassName(String clsName)
        { return clsName.replace('.', '/') + ".class"; }

    public String getClassFileName()
        { return fileNameForClassName(getQualifiedClassName()); }

    public String getSourceName() {
        String src = null;
        loop: for (int i=0; i<numAttributes; i++) {
            if (attributes[i] instanceof SourceFileAttribute) {
                src = ((SourceFileAttribute)attributes[i]).getSourceFile();
                break loop;
        }   }

        if (src != null) {
            String pkg = getPackageName().replace('.', '/');
            if (pkg.length()>0 && !src.startsWith(pkg))
                src = pkg + "/" + src;
            return src;
        }
        return null;
    }
    public long lastModified()
        { return modificationTime; }

    public String getPackageName()
        {   String fullName = getQualifiedClassName();
            if (fullName.indexOf('.') >= 0)
                return fullName.substring(0, fullName.lastIndexOf('.'));
            return "";
        }

    public String getSimpleClassName()
        {   String fullName = getQualifiedClassName();
            if (fullName.indexOf('.') >= 0)
                return fullName.substring(fullName.lastIndexOf('.')+1);
            return fullName;
        }

    public String getQualifiedClassName()
        { return ((CONSTANT_Class)constantAtIndex(thisClass)).getClassName(); }

    public String getSuperClassName()
        { return ((CONSTANT_Class)constantAtIndex(superClass)).getClassName(); }

    public JavaClass getSuperClass()
        { return JavaClass.forName(getSuperClassName()); }

    public JavaClass[] getInterfaces()
        {
            JavaClass[] intfs = new JavaClass[interfaces.length];
            for (int i=0; i<intfs.length; i++)
                intfs[i] = JavaClass.forName(
                    ((CONSTANT_Class)constantAtIndex(interfaces[i])).getClassName()
                );
            return intfs;
        }

    public JavaField [] getDeclaredFields () { return fields; }
    public JavaMethod[] getDeclaredMethods() { return methods; }

    public JavaField [] getAvailableFields () { return null; }
    public JavaMethod[] getAvailableMethods() { return null; }


    int         magic;                  // = 0xCAFEBABE
    int         majorVersion;           // 45
    int         minorVersion;           // 3
    int         numConstants;
    Vector      constantPool;
    int         accessFlags;
    int         thisClass;
    int         superClass;
    int         numInterfaces;
    int         interfaces[];
    int         numFields;
    JavaField   fields[];
    int         numMethods;
    JavaMethod  methods[];
    int         numAttributes;
    AttributeInfo attributes[];  // SourceFileAttribute

    // store the date class was created, if known
    long  modificationTime = -1;    // unknown unless created from file


public int numConstants() { return numConstants; }

public ConstantInfo constantAtIndex(int ix) {
    return (ConstantInfo) constantPool.elementAt(ix);
}

    int indexOf(ConstantInfo con) {
        int ix = constantPool.indexOf(con);
        if (ix < 0) {  // if not found, add it.
            ix = constantPool.size();
            constantPool.addElement( con );
            if ((con instanceof CONSTANT_Long)
             || (con instanceof CONSTANT_Double)) {
                constantPool.addElement( null );
        }   }
        return ix;
    }



/** WARNING!  This returns classes that are distinct from, though
  * identical with, those returned by JavaClass.forName(String).
  * That method maintains a static cache, and therefore the same
  * object will always be returned for a given qualified class name.
  *  This method, however, ignores the cache and creates a new object
  * for the given class name.
  */
public static JavaClass createFrom(File file) {
    InputStream in = null;
    JavaClass cls = null;
    try {
        in = new FileInputStream(file);
        byte[] data = ByteArray.readFully(in);
        cls = createFrom(data);
        cls.modificationTime = file.lastModified();
    } catch (IOException e) {
    } finally {
        if (in!=null) try { in.close(); } catch (Exception e) {}
    }
    return cls;
}

public static JavaClass createFrom(byte[] data) {
    return createFrom(new DataInputStream(new ByteArrayInputStream(data)));
}

public static JavaClass createFrom(DataInputStream in) {
    JavaClass c = new JavaClass();
    try {
        c.magic = in.readInt();
        if (c.magic != 0xCAFEBABE) throw new RuntimeException
            ("Bad magic number!  This is not a class definition.");
        c.majorVersion = in.readUnsignedShort();
        c.minorVersion = in.readUnsignedShort();

        c.numConstants = in.readUnsignedShort();
        c.constantPool = new Vector();
        c.constantPool.addElement( null );
        for (int i=1; i<c.numConstants; i++) {
            ConstantInfo ci = ConstantInfo.createFrom(in, c);
            c.constantPool.addElement( ci );
            if ((ci.tag == ConstantInfo.CONST_Long)
             || (ci.tag == ConstantInfo.CONST_Double)) {
                ++i;
                c.constantPool.addElement( null );
            }
        }

        c.accessFlags = in.readUnsignedShort();
        c.thisClass   = in.readUnsignedShort();
        c.superClass  = in.readUnsignedShort();

        c.numInterfaces = in.readUnsignedShort();
        c.interfaces = new int[c.numInterfaces];
        for (int i=0; i<c.numInterfaces; i++) {
            c.interfaces[i] = in.readUnsignedShort();
        }
        c.numFields = in.readUnsignedShort();
        c.fields = new JavaField[c.numFields];
        for (int i=0; i<c.numFields; i++) {
            c.fields[i] = JavaField.createFrom(in, c);
        }
        c.numMethods = in.readUnsignedShort();
        c.methods = new JavaMethod[c.numMethods];
        for (int i=0; i<c.numMethods; i++) {
            c.methods[i] = JavaMethod.createFrom(in, c);
        }
        c.numAttributes = in.readUnsignedShort();
        c.attributes = new AttributeInfo[c.numAttributes];
        for (int i=0; i<c.numAttributes; i++) {
            c.attributes[i] = AttributeInfo.createFrom(in, c);
        }
    } catch (Exception e) { throw new Error(e.toString()); }
    return c;
}

public void writeTo(DataOutputStream out) {
    try {
        out.writeInt(0xCAFEBABE);
        out.writeShort(majorVersion);
        out.writeShort(minorVersion);

        out.writeShort(numConstants);
        for (int i=1; i<numConstants; i++) {
            ConstantInfo ci = constantAtIndex(i);
            ci.writeTo(out);
            if ((ci.tag == ConstantInfo.CONST_Long)
             || (ci.tag == ConstantInfo.CONST_Double)) {
                ++i;
            }
        }
        out.writeShort(accessFlags);
        out.writeShort(thisClass);
        out.writeShort(superClass);

        out.writeShort(numInterfaces);
        for (int i=0; i<numInterfaces; i++) {
            out.writeShort(interfaces[i]);
        }
        out.writeShort(numFields);
        for (int i=0; i<numFields; i++) {
            fields[i].writeTo(out);
        }
        out.writeShort(numMethods);
        for (int i=0; i<numMethods; i++) {
            methods[i].writeTo(out);
        }
        out.writeShort(numAttributes);
        for (int i=0; i<numAttributes; i++) {
            attributes[i].writeTo(out);
        }
    } catch (Exception e) { throw new Error(e.toString()); }
}
}
