//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.java;
import java.io.*;

public class JavaMethod {
    public AccessFlags getAccessFlags() 
        { return new AccessFlags(accessFlags); }
    
    public JavaClass getDeclaringClass() 
        { return javaClass; }
    
    public String getName()
        { return (String) javaClass.constantAtIndex(nameIndex).getValue(); }
    
    MethodSignature sig = null;
    public MethodSignature getMethodSignature() 
        { if (sig == null) 
            sig = MethodSignature.createFrom((String) javaClass.constantAtIndex(sigIndex).getValue());
          return sig;
        }
    public JavaClass[] getParameters() 
        {
            Signature[] parmSigs = getMethodSignature().getParameters();
            JavaClass[] parms = new JavaClass[parmSigs.length];
            for (int i=0; i<parms.length; i++) 
                parms[i] = JavaClass.forSignature(parmSigs[i]);
            return parms;
        }
    
    public JavaClass getReturnType() 
        { return JavaClass.forSignature(getMethodSignature().getReturnType()); }

    JavaClass javaClass;
    int      accessFlags;  
    int      nameIndex;  
    int      sigIndex;  
    int      numAttributes;  
    AttributeInfo attributes[]; // CodeAttribute 

    static JavaMethod createFrom(DataInputStream in, JavaClass cls) {
        JavaMethod c = new JavaMethod();
        c.javaClass = cls;
        try {
            c.accessFlags   = in.readUnsignedShort();
            c.nameIndex     = in.readUnsignedShort();
            c.sigIndex      = in.readUnsignedShort();
            c.numAttributes = in.readUnsignedShort();
            c.attributes    = new AttributeInfo[c.numAttributes];
            for (int i=0; i<c.numAttributes; i++) 
                c.attributes[i] = AttributeInfo.createFrom(in, cls);
        } catch (Exception e) { throw new Error(e.toString()); }
        return c;
    }
    public void writeTo(DataOutputStream out) {
        try {
            out.writeShort(accessFlags);
            out.writeShort(nameIndex);
            out.writeShort(sigIndex);
            out.writeShort(numAttributes);
            for (int i=0; i<numAttributes; i++) 
                attributes[i].writeTo(out);
        } catch (Exception e) { throw new Error(e.toString()); }
    }
public String toString() { 
    String str = AccessFlags.toString(accessFlags)
        + "  " + getName() 
        + " <" + getMethodSignature().toString() + ">";
    for (int i=0; i<numAttributes; i++) 
        str += "\n" + attributes[i];

    return str; 
}

}
