//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.java;
import java.io.*;
import java.util.*;

public class MethodSignature {
    Signature returnType;
    Signature[] parameters;

    public Signature getReturnType() { return returnType; }

    public Signature[] getParameters() { return parameters; }

static private Vector vec = new Vector();
synchronized public static MethodSignature createFrom(String s) {
    MethodSignature sig = new MethodSignature();
    vec.setSize(0);
    String rtn = s.substring( s.indexOf(')')+1, s.length() );
    sig.returnType = Signature.createFrom(rtn);
    String parms = s.substring( s.indexOf('(')+1, s.indexOf(')') );
    int start = 0;
    while (start < parms.length())  switch (parms.charAt(start)) {
        case 'B': vec.addElement(new Signature("byte"   , true)); ++start; break;
        case 'C': vec.addElement(new Signature("char"   , true)); ++start; break;
        case 'D': vec.addElement(new Signature("double" , true)); ++start; break;
        case 'F': vec.addElement(new Signature("float"  , true)); ++start; break;
        case 'I': vec.addElement(new Signature("int"    , true)); ++start; break;
        case 'J': vec.addElement(new Signature("long"   , true)); ++start; break;
        case 'S': vec.addElement(new Signature("short"  , true)); ++start; break;
        case 'Z': vec.addElement(new Signature("boolean", true)); ++start; break;
        case '[': {
            Signature parm = new Signature();
            String dims = "";
            while (parms.charAt(start) == '[') {
                dims += "[]";
                ++parm.arrayDim;
                ++start;
            }
            switch (parms.charAt(start)) {
                case 'B': parm.setTypeName("byte"    + dims); parm.isBuiltInType = true; ++start; break;
                case 'C': parm.setTypeName("char"    + dims); parm.isBuiltInType = true; ++start; break;
                case 'D': parm.setTypeName("double"  + dims); parm.isBuiltInType = true; ++start; break;
                case 'F': parm.setTypeName("float"   + dims); parm.isBuiltInType = true; ++start; break;
                case 'I': parm.setTypeName("int"     + dims); parm.isBuiltInType = true; ++start; break;
                case 'J': parm.setTypeName("long"    + dims); parm.isBuiltInType = true; ++start; break;
                case 'S': parm.setTypeName("short"   + dims); parm.isBuiltInType = true; ++start; break;
                case 'Z': parm.setTypeName("boolean" + dims); parm.isBuiltInType = true; ++start; break;
                case 'L': {
                    int semi = parms.indexOf(';', start+1);
                    parm.setTypeName(parms.substring(start+1, semi) + dims);
                    parm.isBuiltInType = false;
                    start = semi + 1;
                } break;
                default : throw new Error("Bad method signature '" + parms + "'  " + parms.charAt(start));
            }   
            vec.addElement(parm);
        } break;
        case 'L': {
            int semi = parms.indexOf(';', start+1);
            Signature parm = new Signature();
            parm.setTypeName(parms.substring(start+1, semi));
            parm.isBuiltInType = false;
            vec.addElement(parm);
            start = semi + 1;
        } break;
        default : throw new Error("Bad method signature '" + s + "'"); 
    }
    sig.parameters = new Signature[vec.size()];
    for (int i=0; i<sig.parameters.length; i++) {
        sig.parameters[i] = (Signature) vec.elementAt(i);
    }
    vec.setSize(0);
    return sig;
}

public String toString() {
    String rtn = "(";
    for (int i=0; i<parameters.length; i++) {
        rtn += parameters[i];
        if (i < parameters.length-1)
            rtn += ", ";
    }
    rtn += ") -> ";
    rtn += returnType;
    return rtn;
}
}
