//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.java;
import java.io.*;
import java.util.*;

abstract public class ResourceTree extends Observable {

    public String separator   = "/";    // default branch separator
    public char separatorChar = '/';    // .. as a char

    public static ResourceTree createFrom(String path/*URL url*/) {
        File file = new File(path);

        if (file.isDirectory())
            return new Directory(file);
        else
            return new JarArchive(file);
    }

    protected DirectoryTree dirTree;  // cache the tree for quick lookups

    abstract public String getName();
    abstract public void setName(String name);

    abstract public String[] getDirContents(String qualName);
    abstract public String[] getDirContents(String qualName, FilenameFilter filt);
    abstract public Enumeration childNames();

    abstract public boolean  isDirectory  (String qualName);
    abstract public boolean  contains     (String qualName);
    abstract public long     lastModified (String qualName);
    abstract public byte[]   getBytes     (String qualName);
    abstract public Resource getResource  (String qualName);

    abstract public void     addResource  (Resource r);
    abstract public void    removeResource(String qualName);

    /** create an output stream suitable for updating a child resource
     */
    abstract protected OutputStream getOutputStream(String qualName);

    public String getParent(String qualName)
        { return getParent(qualName, separatorChar);}

    String getParent(String path, char sep) {
        int ix = path.lastIndexOf(sep);
        if (ix == -1) return "";
        if (ix == path.length()-1) { // ends in sep?
         int pix = path.lastIndexOf(sep, ix-1);
         if (pix == -1)
          return "";
         return path.substring(0, pix+1);
        }return path.substring(0,  ix+1);
    }

    /** Returns all contents of the passed directory: both
      * dirs and files.  Returns null if the passed dir is
      * not actually a dir.
      */
    public String[] recursiveList(String qualName) {
        if (!isDirectory(qualName))
            return null;
        Vector list = new Vector();
        Stack stack = new Stack();
        stack.push(qualName);
        while (!stack.empty()) {
            String parent = (String) stack.pop();
            String[] subs = getDirContents(parent);
            if (subs!=null) for (int i=subs.length-1; i>=0; i--) {
                String child = parent + subs[i];
                list.addElement(child);
                if (isDirectory(child))
                    stack.push(child);
        }   }
        String[] subs = new String[list.size()];
        for (int i=0; i<subs.length; i++)
            subs[i] = (String) list.elementAt(i);
        return subs;
    }

    /** has no effect on trees that maintain consistent state
     */
    public void write() {}

    public boolean isModified() { return false; }

    public void notifyObservers() {setChanged(); super.notifyObservers();}
}
