//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.ui;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import java.io.*;
import starlight.util.*;


public class FileSelector
  implements Runnable,
             KeyListener,
             ItemSelectable,
             ActionListener,
             LayoutManager
{
private ActionListener actionListener;
private ItemListener   itemListener;

static public String SELECT = "Open";
static public String CANCEL = "Cancel";

Dialog  dialog;
String  title;
Frame   frame;
boolean frameNotProvided;
boolean userAccepted;       // true when user has accepted the dialog,
                            // false on cancel and before selected.

private Hashtable filters = new Hashtable();

ImageButton upButton        = new ImageButton(null, "folderUp.gif");
Label       pathFieldLbl    = new Label("path: ", Label.RIGHT);
TextField   pathField       = new TextField();
MultiColumnList listPanel   = new MultiColumnList();
Label       fileFieldLbl    = new Label("file: ", Label.RIGHT);
TextField   fileField       = new TextField();
Label       filterChoiceLbl = new Label("filter: ", Label.RIGHT);
Choice      filterChoice    = new Choice();
Button      openButton      = new Button(SELECT);
Button      cancelButton    = new Button(CANCEL);


//---------- Runnable Implementation ---------------------
/**
 * This prevents the caller from blocking on show(), which
 * if this class is used on an awt event thread would
 * cause a deadlock.
 */
public void run() {
    userAccepted = false;
    fileField.requestFocus();
    dialog.setVisible(true);
    fileField.requestFocus();
}

//---------- KeyListener Implementation ------------------
/** Depending on the source of the keystroke, interpret
  * it appropriately (escape always cancels; return or
  * space cancel if originated from cancel button, otherwise
  * select.)
  */
public void    keyTyped(KeyEvent evt) { }
public void keyReleased(KeyEvent evt) { }
public void  keyPressed(KeyEvent evt) {
    if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
        fireActionEvent(CANCEL);
    } else if ((evt.getKeyCode() == KeyEvent.VK_ENTER )
            || (evt.getKeyCode() == KeyEvent.VK_ACCEPT)) {
        if (evt.getSource() == cancelButton)
             fireActionEvent(CANCEL);
        else fireActionEvent(SELECT);
    } else if (evt.getKeyCode() == KeyEvent.VK_SPACE) {
        if (evt.getSource() == cancelButton)
            fireActionEvent(CANCEL);
        else if (evt.getSource() == openButton)
            fireActionEvent(SELECT);
    }
}

//---------- ActionListener Implementation ---------------
/** events which indicate that the dialog is finished and
  * should close filter through here.
  */
public void actionPerformed(ActionEvent evt) {
    // set this first, so when waiting thread wakes up after dialog
    // goes away, it can tell whether dialog results are valid.
    userAccepted = SELECT.equals(evt.getActionCommand());

    dialog.setVisible(false);
    dialog.dispose();
    if (frameNotProvided) frame.dispose();
    if (actionListener != null)
        actionListener.actionPerformed(evt);
}

public void addActionListener(ActionListener listener)
    {   actionListener = AWTEventMulticaster.add(actionListener, listener); }

public void removeActionListener(ActionListener listener)
    {   actionListener = AWTEventMulticaster.remove(actionListener, listener); }

/** fireActionEvent is used to indicate that the dialog is
  * complete, and should close itself.  The actionPerformed()
  * implementation will clean up, and forward the notification
  * on to any interested listeners.  The actionCommand passed
  * will indicate whether the dialog was accepted or cancelled.
  */
public void fireActionEvent(String cmd) {
    actionPerformed (new ActionEvent
        (this, ActionEvent.ACTION_PERFORMED, cmd));
}

//---------- ItemListener Implementation ------------------------------
/**
  * Returns selected objects as an array of File.
  */
public Object[] getSelectedObjects() {
    Object[] items = listPanel.getSelectedObjects();
    if (items==null) return null;
    File[] files = new File[items.length];

    String dir = getDirectory();
    for (int i=0; items!=null && i<items.length; i++)
        files[i] = new File(dir, (String)items[i]);

    return files;
}
public void addItemListener(ItemListener listener)
    { itemListener = AWTEventMulticaster.add(itemListener, listener); }

public void removeItemListener(ItemListener listener)
    { itemListener = AWTEventMulticaster.remove(itemListener, listener); }


//---------- LayoutManager Implementation ---------------
public void addLayoutComponent(String name, Component comp) {}
public void removeLayoutComponent(Component comp) {}

public Dimension   minimumLayoutSize(Container parent) {return preferredLayoutSize(parent);}
public Dimension preferredLayoutSize(Container parent) {
    Dimension dlgSize = getPreferredSize();
    Insets insets = parent.getInsets();
    Dimension clientSize = new Dimension(
        dlgSize.width - insets.left - insets.right,
        dlgSize.height - insets.top - insets.bottom);
    return clientSize;
}
public void layoutContainer(Container parent) {
    int TOP=12, BOTTOM=12, LEFT=16, RIGHT=16, GAPX=8, GAPY=6;
    Insets insets = parent.getInsets();
    Dimension parentSize = parent.getSize();
    Dimension clientSize = new Dimension(
        parentSize.width - insets.left - insets.right,
        parentSize.height - insets.top - insets.bottom);
    Dimension upButtonDim = upButton.getPreferredSize();
    Dimension pathFieldLblDim = pathFieldLbl.getPreferredSize();
    Dimension pathFieldDim = pathField.getPreferredSize();
    Dimension fileFieldLblDim = fileFieldLbl.getPreferredSize();
    Dimension fileFieldDim = fileField.getPreferredSize();
    Dimension filterChoiceLblDim = filterChoiceLbl.getPreferredSize();
    Dimension filterChoiceDim = filterChoice.getPreferredSize();
    Dimension openButtonDim = openButton.getPreferredSize();
    Dimension cancelButtonDim = cancelButton.getPreferredSize();

    int x = insets.left + LEFT, y = insets.top + TOP;
    upButton.setBounds(x, y, upButtonDim.width, upButtonDim.height);

    x += upButtonDim.width + GAPX;
    pathFieldLbl.setBounds(x, y, pathFieldLblDim.width, pathFieldLblDim.height);

    x += pathFieldLblDim.width;
    pathField.setBounds(x, y, clientSize.width-x-RIGHT-GAPX, pathFieldDim.height);

    y += upButtonDim.height + GAPY;
    int listPanelH = clientSize.height-TOP-BOTTOM-4*GAPY
                    -upButtonDim.height-openButtonDim.height-cancelButtonDim.height;
    listPanel.setBounds(insets.left + 6, y, clientSize.width-12, listPanelH);
    y += listPanelH + 2*GAPY;

    int   itemH = Math.max(openButtonDim.height, fileFieldDim.height);
    int  labelW = Math.max(fileFieldLblDim.width, filterChoiceLblDim.width);
    int buttonW = Math.max(openButtonDim.width, cancelButtonDim.width);
    int  fieldW = clientSize.width-LEFT-RIGHT-labelW-buttonW-2*GAPX;

    x = insets.left + LEFT;
    fileFieldLbl.setBounds(x, y,  labelW, itemH); x += labelW;
    fileField   .setBounds(x, y,  fieldW, itemH); x += fieldW + 2*GAPX;
    openButton  .setBounds(x, y, buttonW, itemH);

    x = insets.left + LEFT;
    y += openButtonDim.height + GAPY;
    filterChoiceLbl.setBounds(x, y,  labelW, itemH); x += labelW;
    filterChoice   .setBounds(x, y,  fieldW, itemH); x += fieldW + 2*GAPX;
    cancelButton   .setBounds(x, y, buttonW, itemH);
}


public Dimension getPreferredSize()
    { return new Dimension(400,320); }

void createDialog() {
    if (frame == null) {
        frame = new Frame();
        frameNotProvided = true;
    } else {
        frameNotProvided = false;
    }
    dialog = new Dialog(frame, true); // Modal
    dialog.setTitle(title);
    dialog.setLayout(this);

    dialog.add(upButton);
    dialog.add(pathFieldLbl);
    dialog.add(pathField); pathField.setEditable(false);
    dialog.add(listPanel);
    dialog.add(fileFieldLbl);
    dialog.add(fileField);
    dialog.add(filterChoiceLbl);
    dialog.add(filterChoice);
    dialog.add(openButton);
    dialog.add(cancelButton);

    Color bg = new Color(SystemColor.control    .getRGB());
    Color fg = new Color(SystemColor.controlText.getRGB());
    Color lt = starlight.Colors.atIntensity(bg, 0.9375);
    dialog.setBackground(bg);
    dialog.setForeground(fg);
    listPanel.setBackground(lt);
    listPanel.setForeground(fg);
    fileField.setBackground(bg);
    fileField.setForeground(fg);
    filterChoice.setBackground(lt);
    filterChoice.setForeground(fg);

    dialog.pack();
    centerWindow(dialog);
    initListeners();

    setDirectory("");
    fileField.requestFocus();
}
void initListeners() {
    dialog.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent evt)
            { fireActionEvent(CANCEL); }
    });

    upButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e)
        {   String path = getDirectory();
            File absolute = new File(path);
            String par = absolute.getParent();
            File parent = (par==null)? absolute: new File(par);
            setDirectory(parent.getPath());
        }
    });

    pathField.addTextListener(new TextListener() {
        public void textValueChanged(TextEvent e)
        {   String path = getDirectory();
            File dir = new File(fixupDir(path));
            String[] files = null;
            FilenameFilter filter = getFilenameFilter();
            if (filter == null)
                 files = dir.list();
            else files = dir.list(filter);

            sort(dir, files);

            listPanel.clear();

            for (int i=0; i<files.length; i++) {
                File file = new File(dir, files[i]);
                String itemImage = (file.isDirectory()? "folderClosed.gif": null);
                listPanel.addItem(files[i], itemImage);
            }
            listPanel.validate();
            listPanel.repaint();
            setFile("");
        }
    });

    listPanel.addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent e)
        {   Object[] selected = e.getItemSelectable().getSelectedObjects();

            if (selected == null || selected.length == 0)
                setFile("");
            else if (selected.length == 1)
                setFile((String) selected[0]);
            else {
                String names = quote((String) selected[0]);
                for (int i=1; i<selected.length; i++)
                    names += " " + quote((String) selected[i]);
                setFile(names);
            }
            // also forward the event to any listeners
            if (itemListener != null)
                itemListener.itemStateChanged( new ItemEvent(
                    FileSelector.this, ItemEvent.ITEM_STATE_CHANGED,
                    e.getItem(), e.getStateChange()));
        }
    });
    listPanel.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e)
        {   String name = getFile();
            File sel = null;
            if (name != null && name.length() > 0)
                sel = new File(getDirectory(), name);
            if (sel != null && sel.isDirectory())
                 setDirectory(sel.getAbsolutePath());
            else fireActionEvent(SELECT);
        }
    });

    filterChoice.addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent e)
        {   setDirectory(getDirectory()); } });

    dialog.addKeyListener(this);

    fileField.addActionListener(this);
    fileField.addKeyListener(this);

    openButton.addActionListener(this);
    openButton.addKeyListener(this);

    cancelButton.addActionListener(this);
    cancelButton.addKeyListener(this);
}

private String quote(String s) {
    StringBuffer buf = new StringBuffer(s.length() + 2);
    buf.append('"');
    buf.append(s);
    buf.append('"');
    return buf.toString();
}
private void sort(File dir, String[] list) {
    Collator collator = Collator.getInstance();
    CollationKey[] keys = new CollationKey[list.length];
    for (int i=0; i<list.length; i++)
        keys[i] = collator.getCollationKey(list[i]);

    ArraySorter.sort(keys, new Comparator() {
        public boolean less(Object lhs, Object rhs)
        { return (((CollationKey)lhs).compareTo((CollationKey)rhs) < 0); }
    });
    int marker = 0;
    for (int i=0; i<keys.length; i++) {         // pull out the dirs first...
        String filename = keys[i].getSourceString();
        File file = new File(dir, filename);
        if (file.isDirectory()) {
            list[marker++] = filename;
            keys[i] = null;
    }   }
    for (int i=0; i<keys.length; i++) {        // what's left are the non-dir files.
        if (keys[i] != null)
            list[marker++] = keys[i].getSourceString();
    }
}



public static final int LOAD = FileDialog.LOAD;
public static final int SAVE = FileDialog.SAVE;
private int mode = LOAD;

    //---------- Constructors ------------------------------
    public FileSelector(Frame parent)
        { this(parent, "Select file"); }

    public FileSelector(Frame parent, String title)
        { this(parent, title, LOAD); }

    public FileSelector(Frame parent, String title, int mode)
        {   this.frame = parent;
            this.title = title;
            this.mode = mode;
            createDialog();
        }

    //---------- Public Methods ------------------------------
    public void setMode(int mode)
        { this.mode = mode; }

    public int getMode()
        { return mode; }


    private String fixupDir(String path)
        {   if ((File.separatorChar == '\\')    // fixup Windows bug
             && (path.length() == 3)            // 'C:\'  fails to list;
             && (path.charAt(1) == ':'))        // have to use 'C:\.'
                path += ".";
            return path;
        }
    public void setDirectory(String path)
        {   if (path == null || path.length() == 0)
                path = new File(new File(".").getAbsolutePath()).getParent();
            File newDir = new File(path);
            if (newDir.exists())
                pathField.setText(path);
            else Toolkit.getDefaultToolkit().beep();
        }

    /** Return the Selector's current directory.
      * If current is null, return default cwd.
      */
    public String getDirectory()
        {   String path = pathField.getText();
            if (path == null || path.length() == 0)
                path = new File(new File(".").getAbsolutePath()).getParent();
            File dir = new File(path);
            File canon = null;
            try                   { canon = new File(dir.getCanonicalPath()); }
            catch (IOException e) { canon = new File(dir.getAbsolutePath ()); }
            path = canon.getPath();
            //if (!path.endsWith(File.separator)) path += File.separator;
            return path;
        }


    public void setFile(String file)
        { fileField.setText(file); }

    public String getFile()
        { String file = fileField.getText(); if (file==null) return ""; return file; }


    public int getFilterCount()
        { return filterChoice.getItemCount();  }

    public String getFilterDescription()
        { return filterChoice.getSelectedItem(); }

    public String getFilterDescription(int ix)
        { return filterChoice.getItem(ix);  }

    public FilenameFilter getFilenameFilter()
        { return getFilenameFilter(filterChoice.getSelectedItem());  }

    public FilenameFilter getFilenameFilter(String desc)
        {   if (desc == null) return null;
            return (FilenameFilter) filters.get(desc);
        }
    public void setFilenameFilter(String desc)
        {   filterChoice.select(desc);
            //...trigger event?
        }
    public void addFilenameFilter(String desc, FilenameFilter flt)
        {   int ix = filterChoice.getSelectedIndex();
            filterChoice.add(desc);
            filters.put(desc, flt);
            if (ix == -1) ix = 0;
            if (filterChoice.getSelectedIndex() != ix)
                filterChoice.select(ix);
        }

    public boolean wasAccepted() { return userAccepted; }


    /**
     * Starts the dialog in a separate thread, and returns.
     * Presumably you gave the dialog a listener already, so
     * you'll know when it closes.
     */
    public void show()
        {   // Start a new thread to show the dialog
            new Thread(this).start();
        }

    /**
     * Blocking execute.  Doesn't return until the dialog closes.
     * Don't call this one from an AWT thread, or you'll deadlock.
     */
    public void execute()
        { run(); }



private void centerWindow(Window win) {
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension winDim = win.getSize();
    // If larger than screen, reduce window width or height
    if (screenDim.width < winDim.width) {
        win.setSize(screenDim.width, winDim.height);
    }
    if (screenDim.height < winDim.height) {
        win.setSize(winDim.width, screenDim.height);
    }
    // Center Frame, Dialogue or Window on screen
    int x = (screenDim.width - winDim.width) / 2;
    int y = (screenDim.height - winDim.height) / 2;
    win.setLocation(x, y);
}



public static void main(String[] args) {
    Frame frame = new Frame("testing FilePicker");
    frame.setSize(100,100); frame.pack(); frame.show();
    frame.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) { System.exit(0); }
    });
    MemoryMonitorWindow mem = new MemoryMonitorWindow(frame);

    FileSelector picker = new FileSelector(frame);
    String[] htmlFilters = {"*.htm","*.html"};
    String[]  imgFilters = {"*.gif","*.jpg" };
    picker.addFilenameFilter("All files    (*.*)"         , new FileTypeFilter("*.*"      ));
    picker.addFilenameFilter("Source files (*.java)"      , new FileTypeFilter("*.java"   ));
    picker.addFilenameFilter("HTML files   (*.htm,*.html)", new FileTypeFilter(htmlFilters));
    picker.addFilenameFilter("Image files  (*.gif,*.jpg)" , new FileTypeFilter(imgFilters ));
    picker.addFilenameFilter("Class files  (*.class)"     , new FileTypeFilter("*.class"  ));
    picker.execute();
    String path = picker.getDirectory();
    if (!path.endsWith(File.separator))
        path += File.separator;
    String file = picker.getFile();
    System.out.println("selected: " + path + file);
}
}
