//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.ui;

import java.awt.*;
import java.io.InputStream;

/**
 * Provides an image inside of a canvas, blocking until
 * the image is loaded. Note that a modification could
 * be made to provide optional asynch image loading.
 *
 * The image may be provided or loaded as a resource of a
 * class, which is the usual technique. For example:
 * ImageCanvas canvas = new ImageCanvas(this, "Snazzy.gif");
 *
 * The canvas is sized to fit the image exactly.
 *
 * @author Jack Harich
 */
public class ImageCanvas extends Canvas {

//---------- Private Fields ------------------------------
private Image image;

//---------- Initialization ------------------------------
/**
 * Creates a Canvas containing the Image.
 */
public ImageCanvas(Image image) {
    loadImage(image);
}
/**
 * Creates a Canvas containing an Image loaded from the
 * @param resourceName of @param resourceClass.
 */
public ImageCanvas(Class resourceClass, String resourceName) {
try {
    InputStream resource =
        resourceClass.getResourceAsStream(resourceName);
    byte[] bytes = new byte[resource.available()];
    resource.read(bytes);
    Image image = Toolkit.getDefaultToolkit()
        .createImage(bytes);
    loadImage(image);

} catch(Exception ex) {
    System.out.println("ImageCanvas() - Cannot load '" + resourceName +
"' with class '" + resourceClass.getName() + "'.");
    ex.printStackTrace();
}
} // End method
//---------- Superclass Overrides ------------------------
public void paint(Graphics g) {
    g.drawImage(image,0, 0, this);
}
//---------- Private Methods -----------------------------
private void loadImage(Image image) {

    this.image = image;
    MediaTracker tracker = new MediaTracker(this);
    tracker.addImage(image, 0);

    try {
        tracker.waitForID(0);

    } catch(InterruptedException e){
        System.out.println("Cannot load image");
    }
    setSize(image.getWidth(null), image.getHeight(null));
}


} // End Class
