//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.ui;
import java.awt.*;
import java.awt.event.*;


public class TreePanel
     extends Panel
  implements ItemSelectable,
             ActionListener,
             KeyListener,
             MouseListener
{
static public String ACCEPT = "Select";
static public String CANCEL = "Cancel";

static public Font itemfont = new Font("SansSerif", Font.PLAIN, 11);


    ItemListener   itemListener;
    ActionListener actionListener;

    ScrollPane  scrollPane  = new ScrollPane(ScrollPane.SCROLLBARS_NEVER);
    Scrollbar   scroller    = new Scrollbar(Scrollbar.VERTICAL);
    Panel       itemPanel   = new DoubleBufferPanel();

    protected Branch root = null;

    public TreePanel()
        {   setLayout(new BorderLayout());
            add("Center", scrollPane);
            add("East" , scroller);
            scrollPane.add(itemPanel);
            itemPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

            scroller.addAdjustmentListener(new AdjustmentListener() {
                public void adjustmentValueChanged(AdjustmentEvent e)
                {   int at = scroller.getValue();
                    scrollPane.setScrollPosition(0, at);   }
            });

            setFont(itemfont);
            adjustScroller();
        }

    public void setBackground(Color c)
        {   super.setBackground(c);
            Color lt = starlight.Colors.atIntensity(c, 0.9375);
            itemPanel.setBackground(lt);
        }
    public void setForeground(Color c)
        {   super.setForeground(c);
            double intensity = starlight.Colors.intensity(c);
            if (intensity > 0.6) intensity = 0.6;
            Color fore = starlight.Colors.atIntensity(c, intensity);
            itemPanel.setForeground(fore);
        }

    public void invalidate()
        {   super.invalidate(); repaint(10); }

    protected void validateTree()
        {   super.validateTree();
            adjustScroller();
            if (selected == null)
                selected = new Component[countItems()];
            scrollPane.repaint();
            itemPanel.repaint();
        }

    private void adjustScroller()
        {   Dimension panelDim = itemPanel.getSize();
            Dimension viewable = scrollPane.getViewportSize();
            Point at = scrollPane.getScrollPosition();
            scroller.setValues(at.y, viewable.height, 0, panelDim.height);
            scroller.setUnitIncrement(16);
            scroller.setBlockIncrement(viewable.height-16);
            //scroller.setVisible(panelDim.height > viewable.height);
        }

    public void clear()
        {   itemPanel.removeAll();
            root = null;
            invalidateSelect();
            revalidate();
        }
    void invalidateSelect()
        {   current = -1;
            selected = null;
        }
    void revalidate() {
        Component c = this;
        while (c.getParent()!=null)
            c = c.getParent();
        c.validate();
        c.repaint();
    }

    public Branch getRoot() { return root; }
    public void setRoot(String label, Image imgOpen)
    {   if (root != null)
            itemPanel.remove(root);
        root = addBranch(null, label, imgOpen, imgOpen);
        itemPanel.add(root);
        revalidate();
    }

    public Branch addBranch(Branch parent, String label, Image imgOpen, Image imgClosed)
        {   Branch node = new Branch(parent, label, imgOpen, imgClosed);
            node.addMouseListener(this);
            node.addKeyListener(this);
            invalidateSelect();
            return node;
        }

    public Leaf addLeaf(Branch parent, String label, Image img)
        {   Leaf node = new Leaf(parent, label, img);
            node.addMouseListener(this);
            node.addKeyListener(this);
            invalidateSelect();
            return node;
        }

    public int countItems()
        { return (root==null)? 0: root.countItems(); }

    public Component getSubComponent(int ix)
        { return (root==null)? null: root.getSubComponent(ix); }


    int indexOf(Component item)
        {   int index = 0;
            Container parent = item.getParent();
            while (parent != null && item != root) {
                int itemcount = parent.getComponentCount();
                loop: for (int i=0; i<itemcount; i++) {
                    Component m = parent.getComponent(i);
                    if (m == item) break loop;
                    if (m instanceof Branch)
                        index += ((Branch)m).countItems();
                    else ++index;
                }
                item = parent; parent = parent.getParent();
            }
            if (item==root) return index;
            return -1;
        }

    //---------- ItemSelectable Implementation ---------------------
    /** Returns the selected items as an array of Strings, or null
      * if no items are selected.
      */
    public Object[] getSelectedObjects() {
        if (selected == null)  return null;
        int itemcount = selected.length;
        int selectedcount = 0;
        for (int i=0; i<itemcount; i++) {
            if (selected[i] != null)
                ++selectedcount;
        }
        if (selectedcount == 0)  return null;
        Leaf[] items = new Leaf[selectedcount];
        for (int i=0, curr=0; i<itemcount; i++) {
            if ((selected[i] != null)
             && (selected[i] instanceof Leaf))
                items[curr++] = (Leaf)selected[i];
        }
        return items;
    }

    public void addItemListener(ItemListener listener)
        { itemListener = AWTEventMulticaster.add(itemListener, listener); }

    public void removeItemListener(ItemListener listener)
        { itemListener = AWTEventMulticaster.remove(itemListener, listener); }

    public void fireItemSelectEvent(Component item, boolean selected)
        {   int index = indexOf(item);
            if (selected)
                   select(index);
            else deselect(index);

            if (itemListener != null)
                itemListener.itemStateChanged( new ItemEvent(
                    this, ItemEvent.ITEM_STATE_CHANGED,
                    item, selected? ItemEvent.SELECTED: ItemEvent.DESELECTED));
        }


    //---------- ActionListener Implementation ---------------
    public void actionPerformed(ActionEvent evt)
        {   if (actionListener != null)
                actionListener.actionPerformed(evt);
        }
    public void addActionListener(ActionListener listener)
        { actionListener = AWTEventMulticaster.add(actionListener, listener); }

    public void removeActionListener(ActionListener listener)
        { actionListener = AWTEventMulticaster.remove(actionListener, listener); }

    public void fireActionEvent(String cmd)
        {   actionPerformed(new ActionEvent
                (this, ActionEvent.ACTION_PERFORMED, cmd));
        }


    //---------- KeyListener Implementation ------------------
    public void    keyTyped(KeyEvent evt) { }
    public void keyReleased(KeyEvent evt) { }
    public void  keyPressed(KeyEvent evt) {
        int index = -1;
        switch (evt.getKeyCode()) {
        case KeyEvent.VK_ESCAPE: if (isSelection()) clearSelection(); else fireActionEvent(CANCEL); break;
        case KeyEvent.VK_ENTER :
        case KeyEvent.VK_ACCEPT: fireActionEvent(ACCEPT); break;

        case KeyEvent.VK_LEFT : index =  leftOf(current); break;
        case KeyEvent.VK_RIGHT: index = rightOf(current); break;
        case KeyEvent.VK_UP   : index =   above(current); break;
        case KeyEvent.VK_DOWN : index =   below(current); break;
        case KeyEvent.VK_HOME : index =                0; break;
        case KeyEvent.VK_END  : index =   countItems()-1; break;
        }
        if (index != -1) {
            if (evt.isShiftDown() && evt.isControlDown())
                controlShiftTo(index);
            else if (evt.isShiftDown())
                shiftTo(index);
            else if (evt.isControlDown())
                controlTo(index);
            else goTo(index);
        }
    }

    //---------- MouseListener Implementation ------------------
    public void mouseEntered (MouseEvent e) {}
    public void mouseExited  (MouseEvent e) {}
    public void mousePressed (MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseClicked (MouseEvent e) {
        if (e.getClickCount() == 1) {
            Component comp = (Component) e.getSource();
            int ix = indexOf(comp);
            if (e.isShiftDown() && e.isControlDown())
                controlShiftTo(ix);
            else if (e.isShiftDown())
                shiftTo(ix);
            else if (e.isControlDown())
                controlTo(ix);
            else goTo(ix);
            comp.requestFocus();
        }
        else if (e.getClickCount() == 2)
            fireActionEvent(ACCEPT);
    }

    //-----------------Selection implementation-----------------------
    Component[] selected;
    int current = -1;
    Color selectedBG = new Color(SystemColor.textHighlight    .getRGB());
    Color selectedFG = new Color(SystemColor.textHighlightText.getRGB());

    public boolean isSelection() {
        if (selected == null) return false;
        for (int ix=0; ix<selected.length; ix++)
            if (selected[ix] != null) return true;
        return false;
    }
    public boolean isSelected(Component elt) {
        if (selected == null) return false;
        for (int ix=0; ix<selected.length; ix++)
            if (selected[ix] == elt) return true;
        return false;
    }
    public boolean isSelected(int ix) {
        return (selected != null && ix != -1 && selected[ix] != null);
    }

    public void clearSelection() {
        for (int i=0; i<selected.length; i++)
            if (isSelected(i))
                deselect(i);
        current = -1;
    }

    void select(int index) {
        if (current>=0 && current<selected.length) {
            Leaf prev = (Leaf) getSubComponent(current);
            prev.setFocusBorder(false);
        }
        Leaf comp = (Leaf) getSubComponent(index);
        if (comp != null) {
            selected[index] = comp;
            current = index;
            comp.setBackground(selectedBG);
            comp.setForeground(selectedFG);
            comp.setFocusBorder(true);
            comp.repaint();
        }
    }
    void deselect(int index) {
        if (current>=0 && current<selected.length) {
            Leaf prev = (Leaf) getSubComponent(current);
            prev.setFocusBorder(false);
        }
        Leaf comp = (Leaf) getSubComponent(index);
        if (comp != null) {
            selected[index] = null;
            current = index;
            comp.setBackground(itemPanel.getBackground());
            comp.setForeground(itemPanel.getForeground());
            comp.setFocusBorder(true);
            comp.repaint();
        }
    }
    // reset selection:  clear all but index.
    void goTo(int index) {
        for (int i=0; i<selected.length; i++) {
            if (i != index && isSelected(i))
                fireItemSelectEvent(getSubComponent(i), false);
        }
        fireItemSelectEvent(getSubComponent(index), true);
    }
    // modify selection:  toggle the indicated item.
    void controlTo(int index) {
        fireItemSelectEvent(getSubComponent(index), !isSelected(index));
    }
    // add to selection:  range from current to index.
    void shiftTo(int index) {
        int step = (index<current)? -1: 1;
        for (int i=current; i!=index; i+=step)
            fireItemSelectEvent(getSubComponent(i), true);
        fireItemSelectEvent(getSubComponent(index), true);
    }
    // modify selection:  toggle range from current to index.
    void controlShiftTo(int index) {
        int step = (index<current)? -1: 1;
        for (int i=current; i!=index; i+=step)
            fireItemSelectEvent(getSubComponent(i), !isSelected(i));
        fireItemSelectEvent(getSubComponent(index), !isSelected(index));
    }


    int leftOf(int index) {
        return -1;
    }
    int rightOf(int index) {
        return -1;
    }
    int above(int index) {
        if (index<=0) return -1;
        return index - 1;
    }
    int below(int index) {
        if (index>=selected.length-1) return -1;
        return index + 1;
    }
}
