//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.ui.cmd;
import java.util.*;
import java.awt.event.*;

public class Command 
  implements ActionListener
{
protected Vector invokers = new Vector();
protected CommandHandler handler = null;

private String category;
private String name;

public Command(String fullName) {
    if (fullName.indexOf(':') == -1) throw new RuntimeException
        ("Malformed command name " + fullName);
    this.category = fullName.substring(0, fullName.indexOf(':'));
    this.name     = fullName.substring(fullName.indexOf(':')+1, fullName.length());
}
public Command(String cat, String name) {
    this.category = cat;
    this.name = name;
}

public String categoryName() { return category; }
public String  commandName() { return name; }
public String     fullName() { return category + ":" + name; }


public void   registerInvoker(CommandInvoker ci) { invokers.addElement(ci); }
public void unRegisterInvoker(CommandInvoker ci) { invokers.removeElement(ci); }

/**
*   Associate a handler with a command.  The command will un-hide
*  the menu/toolbar entries, but leave them disabled until the handler
*  calls command.enable();
*/
public void   registerHandler(CommandHandler h) {
    if (handler != null) throw new RuntimeException
        ("Command handler already registered!");
    handler = h;
    install();
    //enable();
}
public void unRegisterHandler(CommandHandler h) {
    //if (handler == null) throw new RuntimeException
    //    ("Can't unregister nonexistent handler!");
    if ((handler != null) && (handler != h)) throw new RuntimeException
        ("Handler is not associated with this command!");
    disable();
    remove();
    handler = null;
}

public void install() {
    for (Enumeration e=invokers.elements(); e.hasMoreElements(); ) {
        CommandInvoker ci = (CommandInvoker) e.nextElement();
        ci.installCommand(this);
    }
}
public void enable() {
    for (Enumeration e=invokers.elements(); e.hasMoreElements(); ) {
        CommandInvoker ci = (CommandInvoker) e.nextElement();
        ci.enableCommand(this);
    }
}
public void disable() {
    for (Enumeration e=invokers.elements(); e.hasMoreElements(); ) {
        CommandInvoker ci = (CommandInvoker) e.nextElement();
        ci.disableCommand(this);
    }
}
public void remove() {
    for (Enumeration e=invokers.elements(); e.hasMoreElements(); ) {
        CommandInvoker ci = (CommandInvoker) e.nextElement();
        ci.removeCommand(this);
    }
}

public Memento execute() {
    if (handler == null) throw new RuntimeException
        ("Can't execute '"+fullName()+"'; no handler registered.");
    return handler.execute();
}
public void unExecute(Memento mem) {
    if (handler == null) throw new RuntimeException
        ("Can't unExecute '"+fullName()+"'; no handler registered.");
    handler.unExecute(mem);
}

public void actionPerformed(ActionEvent e)
    { execute(); }
}
