//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.ui.cmd;
import java.util.*;

abstract public class CommandHandler implements Observer {

Command command;

public CommandHandler(Command command) { this.command = command; install(); }

/**
 * Install the handler by registering it with the command it handles.
 * enable/disable are called to indicate that the menu/button should be
 * enabled/disabled, and they forward the request to the command object.
 */
public void install() { command.  registerHandler(this); }
public void  remove() { command.unRegisterHandler(this); }
public void  enable() { command.enable();  }
public void disable() { command.disable(); }

/**
*   Override this to perform the command.  Return a memento
* object containing any needed state-restoration info, if the
* command is undoable, or null if not.
*/
abstract public Memento execute();

/**
*   If execute is implemented to return a memento, unExecute
*  must be implemented to handle that memento.
*/
public void unExecute(Memento mem) {}

/**
*   Override to determine activation (menu/button graying)
*/
public boolean isInvokable() { return true; }

public void updateInvokers() {
    if (!isInvokable())
        disable();
    else enable();
}
public void update(Observable o, Object arg)
    { updateInvokers(); }
}
