//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.util;

/**
*   Provides encoding of raw bytes to base64, and decoding of base64
*  to raw bytes.
*/
public class Base64 {

static char[] alphabet =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="
    .toCharArray();

static byte[] codes = new byte[256];
static {
    for (int i=0; i<256; i++) codes[i] = -1;
    for (int i = 'A'; i <= 'Z'; i++) codes[i] = (byte)(     i - 'A');
    for (int i = 'a'; i <= 'z'; i++) codes[i] = (byte)(26 + i - 'a');
    for (int i = '0'; i <= '9'; i++) codes[i] = (byte)(52 + i - '0');
    codes['+'] = 62;
    codes['/'] = 63;
}

static public char[] encode(byte[] data) {
    char[] out = new char[((data.length + 2) / 3) * 4];

    for (int i=0, index=0; i<data.length; i+=3, index+=4) {
        boolean quad = false;
        boolean trip = false;

        int val = (0xFF & (int) data[i]);
        val <<= 8;
        if ((i+1) < data.length) {
            val |= (0xFF & (int) data[i+1]);
            trip = true;
        }
        val <<= 8;
        if ((i+2) < data.length) {
            val |= (0xFF & (int) data[i+2]);
            quad = true;
        }
        out[index+3] = alphabet[(quad? (val & 0x3F): 64)];
        val >>= 6;
        out[index+2] = alphabet[(trip? (val & 0x3F): 64)];
        val >>= 6;
        out[index+1] = alphabet[val & 0x3F];
        val >>= 6;
        out[index+0] = alphabet[val & 0x3F];
    }
    return out;
}
static public byte[] decode(char[] data) {
    byte[] out = new byte[((data.length + 3) / 4) * 3];

    int shift = 0;   // # of excess bits stored in accum
    int accum = 0;   // excess bits

    int index = 0;
    boolean finished = false;
    for (int ix=0; ix<data.length && !finished; ix++) {
        int value = codes[ data[ix] & 0xFF ];   // ignore high byte of char
        if ( value != -1 ) {                    // skip over non-code
            accum <<= 6;
            shift += 6;
            if (value == -2)
                finished = true;
            else
                accum |= value;
            if ( shift >= 8 ) {
                shift -= 8;
                out[index++] = (byte) ((accum >> shift) & 0xFF);
                accum &= ~(0xFF << shift);
    }   }   }
    if (index != out.length) {
        byte[] tmp = new byte[index];
        System.arraycopy(out, 0, tmp, 0, index);
        out = tmp;
    }
    return out;
}

public static void main(String[] args) {
    byte[] data = {0, -1, 2, -3, 4, 5, -6, 7, 8, -9, 11, -12, 14, 15 };
    if (args.length == 1) data = args[0].getBytes();
    char[] code = Base64.encode(data);
    byte[] recv = Base64.decode(code);
    System.out.println("data = " + fromBytes(data));
    System.out.println("code = " + fromBytes(code));
    System.out.println("recv = " + fromBytes(recv));
    compare(data, recv);
}
static void compare(byte[] b1, byte[] b2) {
    if (b1 == b2) return;
    if (b1 == null || b2 == null) {
        System.out.println("Null array!");
        return;
    }
    if (b1.length != b2.length) {
        System.out.println("different lengths: " + b1.length + " vs. " + b2.length);
        return;
    }
    for (int i=0; i<b1.length; i++) {
        if (b1[i] != b2[i]) {
            for (int x=0; x<b1.length; x++)
                System.out.print(" " + Integer.toHexString(b1[x]));
            System.out.println("");
            for (int x=0; x<b2.length; x++)
                System.out.print(" " + Integer.toHexString(b2[x]));
            System.out.println("");
            System.out.println("disagree at byte " + i);
            return;
        }
    }
}
static String fromBytes(byte[] data) {
    char[] buf = new char[data.length];
    for (int i=0; i<buf.length; i++)
        buf[i] = (char) data[i];
    return new String(buf);
}
static String fromBytes(char[] data) {
    return new String(data);
}
}
