//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.util;
import java.awt.*;

public class BusyMonitor extends Canvas implements Runnable {
static final int sleepTime = 100;
static final Color green  = Color.green;
static final Color yellow = Color.yellow;
static final Color red    = Color.red;
static final Color darkGray   = Color.darkGray;

public static int prioMin = Thread.MIN_PRIORITY;
public static int prioMax = Thread.MAX_PRIORITY;
             
MonitorThread[] threads;

Image   offscreen;

    public BusyMonitor() {
        setSize(getPreferredSize());
                                 
        threads = new MonitorThread[prioMax-prioMin+1];
        for (int pri=prioMin, ix=0; pri<=prioMax; pri++, ix++) {
            threads[ix] = new MonitorThread(pri);
            threads[ix].start();
        }
        Thread monitor = new Thread(this);
        monitor.setDaemon(true);
        monitor.start();
    }
    public void run() {
        while (true) {
            updateThreadMonitors();
            try { Thread.sleep(sleepTime); } catch(Exception e) {}
        }
    }

    void updateThreadMonitors() {
        if (offscreen == null) {
            Dimension sz = getPreferredSize();
            offscreen = createImage(sz.width, sz.height);
            if (offscreen == null) return;
        }
        int x = border + gap;
        int y = border + gap;
        Graphics g = offscreen.getGraphics(); 
        if (g != null) try {
            Dimension size = getSize();
            g.setColor(getBackground());
            g.clearRect(0,0, size.width, size.height);
            
            for (int i=0; i<threads.length; i++) {
                MonitorThread thread = threads[i];
                int slack = (int) thread.slack();
                if (slack > 1000) slack = 1000;
                int len = barLen * (1000-slack) / 1000;
                Color color = (slack<=333)? green: (slack<=667)? yellow: red;
                g.setColor(darkGray);
                g.fillRect(x, y, barLen, barWid);
                g.setColor(color);
                g.fillRect(x, y, len, barWid);

                y += barWid + gap;
            }
            Graphics screen = getGraphics(); 
            if (screen != null) try {
                screen.drawImage(offscreen, 0, 0, null);
            } finally { screen.dispose(); }
        } finally { g.dispose(); }
    }
    int threadCount() { return prioMax - prioMin + 1; }

    int gap = 4;
    int border = 2;
    int barLen = 32;
    int barWid = 4;
    public Dimension getPreferredSize() {
        int n = threadCount();
        int w = barLen + 2*border + 2*gap;
        int h = barWid*n + 2*border + (n+1)*gap;
        return new Dimension(w, h);
    }
    public void update(Graphics g) { paint(g); }
    public void  paint(Graphics g) {
        Dimension size = getSize();
        g.setColor(getBackground());
        g.clearRect(0,0, size.width, size.height);
    }

    class MonitorThread extends Thread {
        final int sleepTime = 10;
        long lastUpdated = -1;
        int priority = Thread.NORM_PRIORITY;

        MonitorThread(int priority) {
            this.priority = priority; 
            setDaemon(true);
            setPriority(priority);
            setName("PerfCheck " + priority);
        }
        int slack() { return (int) (System.currentTimeMillis()-lastUpdated); }

        public void run() {
            while (true) {
                lastUpdated = System.currentTimeMillis();
                try { sleep(sleepTime); } catch(Exception e) {}
    }   }   }
}
