//////////////////////license & copyright header///////////////////////
//                                                                   //
//              Java Cannery - jar management utility.               //
//                Copyright (c) 1998 by Kevin Kelley                 //
//                                                                   //
// This program is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU General Public License as    //
// published by the Free Software Foundation; either version 2 of    //
// the License, or (at your option) any later version.               //
//                                                                   //
// This program is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     //
// GNU General Public License for more details.                      //
//                                                                   //
// You should have received a copy of the GNU General Public License //
// along with this program in the file 'gpl.html'; if not, write to  //
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,  //
// Boston, MA 02111-1307, USA, or contact the author:                //
//                                                                   //
//                       Kevin Kelley  <kelley@iguana.ruralnet.net>  //
//                                                                   //
////////////////////end license & copyright header/////////////////////
package starlight.util;
import java.io.*;

public class FileTypeFilter implements FilenameFilter {

private String[] types;
private boolean acceptsDirectories = true;

/** construct a filterfor a single type suffix */
public FileTypeFilter(String suffix) {
    this(suffix, true);
}
/** construct a filter with a description, for a set of type suffixes */
public FileTypeFilter(String[] suffix) {
    this(suffix, true);
}
/** construct a filterfor a single type suffix */
public FileTypeFilter(String suffix, boolean acceptsDirectories) {
    types = new String[1];
    types[0] = getSuffix(suffix);
    this.acceptsDirectories = acceptsDirectories;
}
/** construct a filter with a description, for a set of type suffixes */
public FileTypeFilter(String[] suffix, boolean acceptsDirectories) {
    types = suffix;
    for (int i=0; i<types.length; i++) 
        types[i] = getSuffix(types[i]);
    this.acceptsDirectories = acceptsDirectories;
}

/** return true if file ends with appropriate type suffix */
public boolean accept(File dir, String name) {
    String suffix = getSuffix(name);
    if (acceptsDirectories && new File(dir, name).isDirectory()) 
        return true;
    for (int i=0; i<types.length; i++) {
        if (types[i].equals("*")) 
            return true;
        if (suffix.equals(types[i]))
            return true;
    } return false;
}
/** return the suffix:  whatever is after the last '.'. */
String getSuffix(String name) {
    int ix = name.lastIndexOf('.');
    if (ix == -1) return "";
    return name.substring(ix+1);
}
}
